"""
These PREP7 commands are used to define the linear material properties.
"""


def emunit(self, lab="", value="", **kwargs):
    """APDL Command: EMUNIT

    Specifies the system of units for magnetic field problems.

    Parameters
    ----------
    lab
        Label specifying the type of units:

        MKS - Rationalized MKS system of units (meters, amperes, henries, webers, etc.).
              Free-space permeability is set to 4 πe-7 henries/meter. Free-
              space permittivity is set to 8.85 e-12 F/m.

        MUZRO - User defined system of units. Free-space permeability is set to the value input
                for VALUE. Other units must correspond to the permeability
                units. Relative permeability may be altered to absolute
                values.

        EPZRO - User defined system of units. Free-space permittivity is set to the value input
                for VALUE. Other units must correspond to the permittivity
                units.

    value
        User value of free-space permeability (defaults to 1) if Lab =
        MUZRO, or free-space permittivity (defaults to 1) if Lab = EPZRO.

    Notes
    -----
    Specifies the system of units to be used for electric and magnetic
    field problems. The free-space permeability and permittivity values may
    be set as desired. These values are used with the relative property
    values [MP] to establish absolute property values.

    Note:: : If the magnetic source field strength (Hs) has already been
    calculated [BIOT], switching EMUNIT will not change the values.

    For micro-electromechanical systems (MEMS), where dimensions are on the
    order of microns, see the conversion factors in System of Units in the
    Coupled-Field Analysis Guide.

    This command is also valid in SOLUTION.
    """
    command = "EMUNIT,%s,%s" % (str(lab), str(value))
    return self.run(command, **kwargs)


def mp(self, lab="", mat="", c0="", c1="", c2="", c3="", c4="", **kwargs):
    """APDL Command: MP

    Defines a linear material property as a constant or a function of
    temperature.

    Parameters
    ----------
    lab
        Valid material property label.  Applicable labels are listed under
        "Material Properties" in the input table for each element type in
        the Element Reference.  See Linear Material Properties in the
        Material Reference for more complete property label definitions:

        ALPD - Mass matrix multiplier for damping.

        ALPX - Secant coefficients of thermal expansion (also ALPY, ALPZ).

        BETD - Stiffness matrix multiplier for damping.

        Note:If used in an explicit dynamic analysis, the value corresponds to the percentage of damping in the high frequency domain. For example, 0.1 roughly corresponds to 10% damping in the high frequency domain. - BETX

        Coefficient of diffusion expansion (also BETY, BETZ) - BVIS

        Bulk viscosity - C

        Specific heat - CREF

        Reference concentration (may not be temperature dependent) - CSAT

        Saturated concentration - CTEX

        Instantaneous coefficients of thermal expansion (also CTEY, CTEZ) - CVH

        Heat coefficient at constant volume per unit of mass - DENS

        Mass density. - DMPR

        Constant structural damping coefficient in full harmonic analysis or damping ratio in mode-superposition analysis. - DXX

        Diffusivity coefficients (also DYY, DZZ) - EMIS

        Emissivity. - ENTH

        Enthalpy. - EX

        Elastic moduli (also EY, EZ) - GXY

        Shear moduli (also GYZ, GXZ) - HF

        Convection or film coefficient - KXX

        Thermal conductivities (also KYY, KZZ) - LSST

        Electric loss tangent - LSSM

        Magnetic loss tangent - MGXX

        Magnetic coercive forces (also MGYY, MGZZ) - MURX

        Magnetic relative permeabilities (also MURY, MURZ) - MU

        Coefficient of friction - NUXY

        Minor Poisson's ratios (also NUYZ, NUXZ) (NUXY = νyx, as described in Stress-Strain Relationships in the Mechanical APDL Theory Reference) - PERX

        Electric relative permittivities (also PERY, PERZ) - Note:  If you enter permittivity values less than 1 for SOLID5, PLANE13, or
                          SOLID98, the program interprets the values as
                          absolute permittivity. Values input for PLANE223,
                          SOLID226, or SOLID227 are always interpreted as
                          relative permittivity.

        PRXY - Major Poisson's ratios (also PRYZ, PRXZ) (PRXY = νxy, as described in Stress-
               Strain Relationships in the Mechanical APDL Theory
               Reference)

        QRATE - Heat generation rate for thermal mass element MASS71. Fraction of plastic work
                converted to heat (Taylor-Quinney coefficient) for coupled-
                field elements PLANE223, SOLID226, and SOLID227.

        REFT - Reference temperature.  Must be defined as a constant; C1 through C4 are
               ignored.

        RH - Hall Coefficient.

        RSVX - Electrical resistivities (also RSVY, RSVZ).

        SBKX - Seebeck coefficients (also SBKY, SBKZ).

        SONC - Sonic velocity.

        THSX - Thermal strain (also THSY, THSZ).

        VISC - Viscosity.

    mat
        Material reference number to be associated with the elements
        (defaults to the current MAT setting [MAT]).

    c0
        Material property value, or if a property-versus-temperature
        polynomial is being defined, the constant term in the polynomial.
        C0 can also be a table name (%tabname%); if C0 is a table name, C1
        through C4 are ignored.

    c1, c2, c3, c4
        Coefficients of the linear, quadratic, cubic, and quartic terms,
        respectively, in the property-versus-temperature polynomial.  Leave
        blank (or set to zero) for a constant material property.

    Notes
    -----
    MP defines a linear material property as a constant or in terms of a
    fourth order polynomial as a function of temperature. (See the TB
    command for nonlinear material property input.) Linear material
    properties typically require a single substep for solution, whereas
    nonlinear material properties require multiple substeps;  see Linear
    Material Properties in the Material Reference for details.

    If the constants C1 - C4 are input, the polynomial

    Property = C0 + C1(T) + C2(T)2 + C3(T)3 + C4(T)4

    is evaluated at discrete temperature points with linear interpolation
    between points (that is, a piecewise linear representation) and a
    constant-valued extrapolation beyond the extreme points. First-order
    properties use two discrete points (±9999°). The MPTEMP or MPTGEN
    commands must be used for second and higher order properties to define
    appropriate temperature steps. To ensure that the number of
    temperatures defined via the MPTEMP and MPTGEN commands is minimally
    sufficient for a reasonable representation of the curve, ANSYS
    generates an error message if the number is less than N, and a warning
    message if the number is less than 2N. The value N represents the
    highest coefficient used; for example, if C3 is nonzero and C4 is zero,
    a cubic curve is being used which is defined using 4 coefficients so
    that N = 4.
    """
    command = "MP,%s,%s,%s,%s,%s,%s,%s" % (str(lab), str(
        mat), str(c0), str(c1), str(c2), str(c3), str(c4))
    return self.run(command, **kwargs)


def mpamod(self, mat="", deftemp="", **kwargs):
    """APDL Command: MPAMOD

    Modifies temperature-dependent secant coefficients of thermal
    expansion.

    Parameters
    ----------
    mat
        Material number for which the secant coefficients of thermal
        expansion (SCTE's) are to be modified.  Defaults to 1.

    deftemp
        Definition temperature at which the existing SCTE-versus-
        temperature tables were defined.  Defaults to zero.

    Notes
    -----
    This command converts temperature-dependent SCTE data (properties ALPX,
    ALPY, ALPZ)  from the definition temperature (DEFTEMP) to the reference
    temperature defined by MP,REFT or TREF.  If both the MP,REFT and TREF
    commands have been issued, the reference temperature defined by the
    MP,REFT command will be used.

    This command does not apply to the instantaneous coefficients of
    thermal expansion (properties CTEX, CTEY, CTEZ) or to the thermal
    strains (properties THSX, THSY, THSZ).

    See Linear Material Properties in the Mechanical APDL Material
    Reference and the Mechanical APDL Theory Reference for more details.

    This command is also valid in SOLUTION.
    """
    command = "MPAMOD,%s,%s" % (str(mat), str(deftemp))
    return self.run(command, **kwargs)


def mpchg(self, mat="", elem="", **kwargs):
    """APDL Command: MPCHG

    Changes the material number attribute of an element.

    Parameters
    ----------
    mat
        Assign this material number to the element.  Material numbers are
        defined with the material property commands [MP].

    elem
        Element for material change.  If ALL, change materials for all
        selected elements [ESEL].

    Notes
    -----
    Changes the material number of the specified element.  Between load
    steps in SOLUTION, material properties cannot be changed from linear to
    nonlinear, or from one nonlinear option to another.

    If you change from one MKIN model to another MKIN model, the different
    MKIN models need to have the same number of data points. This
    requirement also applies if you change from one KINH model to another
    KINH model, or from one CHABOCHE model to another CHABOCHE model.
    """
    command = "MPCHG,%s,%s" % (str(mat), str(elem))
    return self.run(command, **kwargs)


def mpcopy(self, matf="", matt="", **kwargs):
    """APDL Command: MPCOPY

    Copies linear material model data from one material reference number to
    another.

    Parameters
    ----------
    matf
        Material reference number from where material property data will be
        copied.

    matt
        Material reference number to where material property data will be
        copied.

    Notes
    -----
    The MPCOPY command copies linear material properties only, which are
    all properties defined through the MP command. If you copy a model that
    includes both linear and yield behavior constants (for example, a BKIN
    model), the MPCOPY and TBCOPY, ALL commands are used together to copy
    the entire model.  All input data associated with the model is copied,
    that is, all data defined through the MP and TB commands.

    Also, if you copy a material model using the Material Model Interface
    (Edit> Copy), both the commands MPCOPY and TBCOPY, ALL are issued,
    regardless of whether the model includes linear constants only, or if
    it includes a combination of linear and yield behavior constants.

    This command is also valid in SOLUTION.
    """
    command = "MPCOPY,%s,%s" % (str(matf), str(matt))
    return self.run(command, **kwargs)


def mpdata(self, lab="", mat="", sloc="", c1="", c2="", c3="", c4="",
           c5="", c6="", **kwargs):
    """APDL Command: MPDATA

    Defines property data to be associated with the temperature table.

    Parameters
    ----------
    lab
        Valid property label.  Applicable labels are listed under "Material
        Properties" in the input table for each element type in the Element
        Reference.  See Linear Material Properties in the Mechanical APDL
        Material Reference for more complete property label definitions:

        ALPD - Mass matrix multiplier for damping.

        ALPX - Secant coefficients of thermal expansion (also ALPY, ALPZ).  (See also MPAMOD
               command for adjustment to reference temperature).

        BETD - Stiffness matrix multiplier for damping.

        BETX - Coefficient of diffusion expansion (also BETY, BETZ)

        C - Specific heat.

        CREF - Reference concentration (may not be temperature dependent)

        CSAT - Saturated concentration

        CTEX - Instantaneous coefficients of thermal expansion (also CTEY, CTEZ).

        DENS - Mass density.

        DMPR - Constant material damping coefficient.

        DXX - Diffusivity coefficients (also DYY, DZZ)

        EMIS - Emissivity.

        ENTH - Enthalpy.

        EX - Elastic moduli (also EY, EZ).

        GXY - Shear moduli (also GYZ, GXZ).

        HF - Convection or film coefficient.

        KXX - Thermal conductivities (also KYY, KZZ).

        LSST - Dielectric loss tangent.

        MGXX - Magnetic coercive forces (also MGYY, MGZZ).

        MU - Coefficient of friction.

        MURX - Magnetic relative permeabilities (also MURY, MURZ).

        NUXY - Minor Poisson's ratios (also NUYZ, NUXZ).

        PERX - Electric relative permittivities (also PERY, PERZ).

        PRXY - Major Poisson's ratios (also PRYZ, PRXZ).

        QRATE - Heat generation rate.

        REFT - Reference temperature (may not be temperature dependent).

        RH - Hall Coefficient.

        RSVX - Electrical resistivities (also RSVY, RSVZ).

        SBKX - Seebeck coefficients (also SBKY, SBKZ).

        SONC - Sonic velocity.

        THSX - Thermal strain (also THSY, THSZ).

        VISC - Viscosity.

    mat
        Material reference number to be associated with the elements
        (defaults to 1 if you specify zero or no material number).

    sloc
        Starting location in table for generating data.  For example, if
        SLOC = 1, data input in the C1 field is the first constant in the
        table.  If SLOC = 7, data input in the C1 field is the seventh
        constant in the table, etc.  Defaults to the last location filled +
        1.

    c1, c2, c3, . . . , c6
        Property data values assigned to six locations starting with SLOC.
        If a value is already in this location, it is redefined.  A blank
        (or zero) value for C1 resets the previous value in SLOC to zero.
        A value of zero can only be assigned by C1.  Blank (or zero) values
        for C2 to C6 leave the corresponding previous values unchanged.

    Notes
    -----
    Defines a table of property data to be associated with the temperature
    table.  Repeat MPDATA command for additional values (100 maximum).
    Temperatures must be defined first [MPTEMP].  Also stores assembled
    property function table (temperature and data) in virtual space.

    This command is also valid in SOLUTION.

    Without Emag enabled, the MUR_ and MG__ properties are not allowed.  In
    ANSYS Professional, all structural and thermal properties are allowed
    except ALPD, BETD, and MU.  In ANSYS Emag, only the RSV_, PER_, MUR_,
    and MG__ properties are allowed. Only products that include ANSYS Emag
    can use the LSST property. The SBK_ property is only available in ANSYS
    Multiphysics and ANSYS PrepPost.
    """
    command = "MPDATA,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(lab), str(mat), str(
        sloc), str(c1), str(c2), str(c3), str(c4), str(c5), str(c6))
    return self.run(command, **kwargs)


def mpdele(self, lab="", mat1="", mat2="", inc="", lchk="", **kwargs):
    """APDL Command: MPDELE

    Deletes linear material properties.

    Parameters
    ----------
    lab
        Material property label (see MP command for valid labels).  If ALL,
        delete properties for all applicable labels.

    mat1, mat2, inc
        Delete materials from MAT1 to MAT2 (defaults to MAT1) in steps of
        INC (defaults to 1).  If MAT1 = ALL, MAT2 and INC are ignored and
        the properties for all materials are deleted.

    lchk
        Specifies the level of element-associativity checking:

        NOCHECK - No element-associativity check occurs. This option is the default.

        WARN - When a section, material, or real constant is associated with an element, ANSYS
               issues a message warning that the necessary entity has been
               deleted.

        CHECK - The command terminates, and no section, material, or real constant is deleted
                if it is associated with an element.

    Notes
    -----
    This command is also valid in SOLUTION.

    The LCHK argument is valid only when Lab = ALL.
    """
    command = "MPDELE,%s,%s,%s,%s,%s" % (
        str(lab), str(mat1), str(mat2), str(inc), str(lchk))
    return self.run(command, **kwargs)


def mpdres(self, labf="", matf="", labt="", matt="", **kwargs):
    """APDL Command: MPDRES

    Reassembles existing material data with the temperature table.

    Parameters
    ----------
    labf
        Material property label associated with MATF.

    matf
        Material reference number of property to restore from virtual
        space.

    labt
        Material property label associated with MATT (defaults to label
        associated with MATF).

    matt
        Material reference number assigned to generated property (defaults
        to MATF).

    Notes
    -----
    Restores into the database (from virtual space) a data table previously
    defined [MP] for a particular property, assembles data with current
    database temperature table, and stores back in virtual space as a new
    property.

    This command is also valid in SOLUTION.
    """
    command = "MPDRES,%s,%s,%s,%s" % (
        str(labf), str(matf), str(labt), str(matt))
    return self.run(command, **kwargs)


def mplib(self, r_w_opt="", path="", **kwargs):
    """APDL Command: /MPLIB

    Sets the default material library read and write paths.

    Parameters
    ----------
    r-w_opt
        Determines what path is being set.  Possible values are:

        READ - Set the read path.

        WRITE - Set the write path.

        STAT - Report what read and write paths are currently in use.

    path
        The directory path to be used for material library files.

    Notes
    -----
    The /MPLIB command sets two path strings used in conjunction with the
    material library feature and the MPREAD and MPWRITE commands.

    For MPREAD, when you use the LIB option and no directory path is given
    in the file name, the command searches for the file in these locations:
    the current working directory, the user's home directory, the user-
    specified material library directory (as defined by the
    /MPLIB,READ,PATH command), and /ansys_dir/matlib.

    For MPWRITE, when you use the LIB option and the directory portion of
    the specification for the material library file is blank, the command
    writes the material  library file to the directory specified by the
    /MPLIB,WRITE,PATH command (if that path has been set).  If the path has
    not been set, the default is to write the file to the current working
    directory.

    The Material Library files supplied with the distribution disks are
    meant for demonstration purposes only.  These files are not intended
    for use in customer applications.
    """
    command = "/MPLIB,%s,%s" % (str(r_w_opt), str(path))
    return self.run(command, **kwargs)


def mplist(self, mat1="", mat2="", inc="", lab="", tevl="", **kwargs):
    """APDL Command: MPLIST

    Lists linear material properties.

    Parameters
    ----------
    mat1, mat2, inc
        List materials from MAT1 to MAT2 (defaults to MAT1) in steps of INC
        (defaults to 1).  If MAT1= ALL (default), MAT2 and INC are ignored
        and properties for all material numbers are listed.

    lab
        Material property label (see the MP command for labels).  If ALL
        (or blank), list properties for all labels.  If EVLT, list
        properties for all labels evaluated at TEVL.

    tevl
        Evaluation temperature for Lab = EVLT listing (defaults to BFUNIF).

    Notes
    -----
    For Lab = EVLT, when the property is from tables, the MPPLOT command
    will not be valid because the property could be a function of more than
    temperature.

    This command is valid in any processor.
    """
    command = "MPLIST,%s,%s,%s,%s,%s" % (
        str(mat1), str(mat2), str(inc), str(lab), str(tevl))
    return self.run(command, **kwargs)


def mpplot(self, lab="", mat="", tmin="", tmax="", pmin="", pmax="",
           **kwargs):
    """APDL Command: MPPLOT

    Plots linear material properties as a function of temperature.

    Parameters
    ----------
    lab
        Linear material property label (EX, EY, etc.) [MP].

    mat
        Material reference number. Defaults to 1.

    tmin
        Minimum abscissa value to be displayed.

    tmax
        Maximum abscissa value.

    pmin
        Minimum property (ordinate) value to be displayed.

    pmax
        Maximum property value.

    Notes
    -----
    When the property is from tables, the MPPLOT command will not be valid
    because the property could be a function of more than temperature.

    This command is valid in any processor.
    """
    command = "MPPLOT,%s,%s,%s,%s,%s,%s" % (str(lab), str(
        mat), str(tmin), str(tmax), str(pmin), str(pmax))
    return self.run(command, **kwargs)


def mpread(self, fname="", ext="", lib="", **kwargs):
    """APDL Command: MPREAD

    Reads a file containing material properties.

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including directory). If you do not specify the LIB
        option, the default directory is the current working
        directory. If you specify the LIB option, the default is
        the following search path: the current working directory,
        the user's home directory, MPLIB_DIR (as specified by the
        /MPLIB,READ,PATH command) and /ansys_dir/matlib (as
        defined by installation). If you use the default for your
        directory, you can use all 248 characters for the file
        name.

    ext
        Filename extension (eight-character maximum).

    lib
        Reads material library files previously written with the
        MPWRITE command.  (See the description of the LIB option
        for the MPWRITE command.)  The only allowed value for LIB
        is LIB.

    Notes
    -----
    Material properties written to a file without the LIB option
    do not support nonlinear properties.  Also, properties written
    to a file without the LIB option are restored in the same
    material number as originally defined.  To avoid errors, use
    MPREAD with the LIB option only when reading files written
    using MPWRITE with the LIB option.

    If you omit the LIB option for MPREAD, this command supports
    only linear properties.

    Material numbers are hardcoded.  If you write a material file
    without specifying the LIB option, then read that file in
    using the MPREAD command with the LIB option, the ANSYS
    program will not write the file to a new material number.
    Instead, it will write the file to the "old" material number
    (the number specified on the MPWRITE command that created the
    file.)

    This command is also valid in SOLUTION.
    """
    return self.run(f"MPREAD,{fname},{ext},,{lib}", **kwargs)


def mptemp(self, sloc="", t1="", t2="", t3="", t4="", t5="", t6="",
           **kwargs):
    """APDL Command: MPTEMP

    Defines a temperature table for material properties.

    Parameters
    ----------
    sloc
        Starting location in table for entering temperatures.  For example,
        if SLOC = 1, data input in the T1 field applies to the first
        constant in the table.  If SLOC = 7, data input in the T1 field
        applies to the seventh constant in the table, etc.  Defaults to the
        last location filled + 1.

    t1, t2, t3, . . . , t6
        Temperatures assigned to six locations starting with SLOC.  If a
        value is already in this location, it will be redefined.  A blank
        (or zero) value for T1 resets the previous value in SLOC to zero.
        A value of zero can only be assigned by T1.  Blank (or zero) values
        for T2 to T6 leave the corresponding previous values unchanged.

    Notes
    -----
    Defines a temperature table to be associated with the property data
    table [MPDATA].  These temperatures are also used for polynomial
    property evaluation, if defined [MP].  Temperatures must be defined in
    non-descending order.  Issue MATER $ STAT to list the current
    temperature table.  Repeat MPTEMP command for additional temperatures
    (100 maximum).  If all arguments are blank, the temperature table is
    erased.

    For clear definition, the temperature range you define with the MPTEMP
    command should include the entire range you'll use in subsequently
    defined materials.  To assist the user in this, the first (and only the
    first) excursion out of the temperature range defined by the MPTEMP
    commands is flagged with a warning message.  Similarly, the reference
    temperature (TREF or MP,reft commands) should also fall in this same
    temperature range.  If not and MP,alpx was used, a note will be output.
    If not, and MP,ctex or MP,thsx was used, an error message will be
    output.

    This command is also valid in SOLUTION.
    """
    command = "MPTEMP,%s,%s,%s,%s,%s,%s,%s" % (
        str(sloc), str(t1), str(t2), str(t3), str(t4), str(t5), str(t6))
    return self.run(command, **kwargs)


def mptgen(self, stloc="", num="", tstrt="", tinc="", **kwargs):
    """APDL Command: MPTGEN

    Adds temperatures to the temperature table by generation.

    Parameters
    ----------
    stloc
        Starting location in table for generating temperatures.  Defaults
        to last location filled + 1.

    num
        Number of temperatures to be generated (1-100).

    tstrt
        Temperature assigned to STLOC location.

    tinc
        Increment previous temperature by TINC and assign to next location
        until all NUM locations are filled.

    Notes
    -----
    Adds temperatures to the temperature table by generation.  May be used
    in combination (or in place of) the MPTEMP command.

    This command is also valid in SOLUTION.
    """
    command = "MPTGEN,%s,%s,%s,%s" % (
        str(stloc), str(num), str(tstrt), str(tinc))
    return self.run(command, **kwargs)


def mptres(self, lab="", mat="", **kwargs):
    """APDL Command: MPTRES

    Restores a temperature table previously defined.

    Parameters
    ----------
    lab
        Material property label [MP].

    mat
        Material reference number.

    Notes
    -----
    Restores into the database (from virtual space) a temperature table
    previously defined [MP] for a particular property.  The existing
    temperature table in the database is erased before this operation.

    This command is also valid in SOLUTION.
    """
    command = "MPTRES,%s,%s" % (str(lab), str(mat))
    return self.run(command, **kwargs)


def mpwrite(self, fname="", ext="", lib="", mat="", **kwargs):
    """APDL Command: MPWRITE

    Writes linear material properties in the database to a file
    (if the LIB option is not specified) or writes both linear and
    nonlinear material properties (if LIB is specified) from the
    database to a file.

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including
        directory). If you do not specify the LIB option, the default
        directory is the current working directory. If you specify LIB and
        you have specified a material library directory (via the /MPLIB
        command), that directory is the default. Otherwise, the default is
        the current working directory. If you use the default for your
        directory, you can use all 248 characters for the file name.

        The file name defaults to Jobname.

    ext
        Filename extension (eight-character maximum).

        If you omit the LIB option, the default extension is
        MP. If you specify the LIB option, the default extension
        is units_MPL, where units is the system of units currently
        in use. (See the description of the /UNITS command.) For
        example, if /UNITS is set to BIN, the extension defaults
        to BIN_MPL.

    lib
        The only value allowed for this field is the string "LIB."

        The LIB option indicates that you wish to have properties
        associated with the material (MAT) written to the
        specified material library file using the material library
        file format. The material library file format is
        ASCII-text-based ANSYS command input. Certain commands
        associated with this format have been modified to
        interpret the string "_MATL" to mean the currently
        selected material. This feature makes the material library
        file independent of the material number in effect when the
        file was written; this enables you to restore the
        properties into the ANSYS database using the material
        number of your choice. The LIB option also enables you to
        save both linear and nonlinear properties. If you omit the
        LIB option, you can save linear properties only.

    mat
        Specifies the material to be written to the named material library
        file.  There is no default; you must either specify a material or
        omit the MAT argument.  Even if you specify a MAT value, the ANSYS
        program ignores it if the LIB argument is not specified.

    Notes
    -----
    Writes linear material properties currently in the database to a file.
    The file is rewound before and after writing.

    This command is also valid in SOLUTION.
    """
    return self.run(f"MPWRITE,{fname},{ext},,{lib},{mat}", **kwargs)


def tbft(self, oper="", id_="", option1="", option2="", option3="",
         option4="", option5="", option6="", option7="", **kwargs):
    """APDL Command: TBFT

    Performs material curve-fitting operations.

    Parameters
    ----------
    oper
        The specific curve-fitting operation:

        Define a constitutive model. - Delete a constitutive model.

        Write data related to a constitutive model to the database (same as TB command). - Initialize coefficients of a constitutive model for nonlinear curve-fitting
                          procedure.

        Deletes coefficients at current reference temperature. Applicable only for temperature dependent coefficients.  - Solve for coefficients.

        Fix (hold constant) the coefficient you specify in Option4. - Add experimental data.

        Delete experimental data. - List all data associated with the material model represented by the material ID
                          number.

    id_
        The material reference number (same as MAT argument used in the TB
        command). Valid entry is any number greater than zero (default = 1)
        but less than 100,000.

    option1
        For curve-fit function operations (Oper = FADD, FDEL, FSET, SET,
        CDEL, SOLVE or FIX) this field specifies the category (HYPER).

    option2
        For curve-fit function operations (Oper = FADD, FDEL, FSET, SET,
        CDEL, SOLVE, or FIX), this field specifies constitutive model type.
        The valid entries are listed in Table 231: Hyperelastic Options
        below.

    option3
        For Oper = FADD, FDEL, FSET, CDEL, SET, SOLVE or FIX, some of the
        cases specified in Option2 will require that the polynomial order
        be specified. The applicable values for the order specification are
        listed in Table 231: Hyperelastic Options.

    option4
        When you are working on a specific coefficient (Oper = FIX), this
        field specifies the index of that coefficient. Valid entries vary
        from 1 to n, where n is the total number of coefficients (default =
        1).

    option5
        When you are working on a specific coefficient (Oper = FIX), this
        field specifies the index of that coefficient. Valid entries vary
        from 1 to N, where N is the total number of coefficients (default =
        1)

    option6
        If Oper = SOLVE, specifies the allowed tolerance in residual change
        to stop an iteration. Valid entry is 0.0 to 1.0 (default = 0.0).

    option7
        If Oper = SOLVE, specifies the allowed tolerance in coefficient
        change to stop an iteration. Valid entry is 0 to 1 (default = 0).
    """
    command = "TBFT,%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(oper), str(id_), str(option1), str(
        option2), str(option3), str(option4), str(option5), str(option6), str(option7))
    return self.run(command, **kwargs)


def uimp(self, mat="", lab1="", lab2="", lab3="", val1="", val2="",
         val3="", **kwargs):
    """APDL Command: UIMP

    Defines constant material properties (GUI).

    Parameters
    ----------
    mat
        Material number.

    lab1, lab2, lab3
        Material property labels (see the MP command for valid labels).

    val1, val2, val3
        Values corresponding to three labels.

    Notes
    -----
    Defines constant material properties.  This is a command generated by
    the Graphical User Interface (GUI) and will appear in the log file
    (Jobname.LOG) if material properties are specified using the Material
    Properties dialog box. This command is not intended to be typed in
    directly in an ANSYS session (although it can be included in an input
    file for batch input or for use with the /INPUT command).
    """
    command = "UIMP,%s,%s,%s,%s,%s,%s,%s" % (str(mat), str(
        lab1), str(lab2), str(lab3), str(val1), str(val2), str(val3))
    return self.run(command, **kwargs)



"""
These DATABASE commands are generated by the GUI when picking
operations are performed.  They can also be used to add additional
parameters to a command.  For example, more points to a spline than
would be normally supported.
"""


def fitem(self, nfield="", item="", itemy="", itemz="", **kwargs):
    """Identifies items chosen by a picking operation (GUI).

    APDL Command: FITEM

    Parameters
    ----------
    nfield
        Field number on the command which uses the picking data.  (Count
        the command name as a field, so that a 2 indicates the first
        command argument, 3 the second command argument, etc.)  The
        corresponding field on the command will have a P51X label.

    item
        Entity number of the entity picked.  Negative entity numbers are
        used to indicate a range of entities.  If the item picked is a
        coordinate location, then this field represents the X-coordinate.
        See also the FLST command.

    itemy, itemz
        Y and Z coordinates of a picked coordinate location.  ITEM
        represents the X coordinate.  See also the FLST  command.

    Notes
    -----
    This is a command generated by the GUI and will appear in the log file
    (Jobname.LOG) if graphical picking is used.  This command is not
    intended to be typed in directly in an ANSYS session (although it can
    be included in an input file for batch input or for use with the /INPUT
    command).

    On the log file, a set of FITEM commands is preceded by one FLST
    command which defines the picking specifications for that pick
    operation.  The data listed in the FITEM commands are used by the first
    subsequent command containing a P51X label in one of its fields.

    Caution:: : For a given entity type, a list containing an ITEM that is
    larger than the maximum defined entity, could deplete the system memory
    and produce unpredictable results.

    This command is valid in any processor.
    """
    command = "FITEM,%s,%s,%s,%s" % (
        str(nfield), str(item), str(itemy), str(itemz))
    return self.run(command, **kwargs)


def flst(self, nfield="", narg="", type_="", otype="", leng="", **kwargs):
    """Specifies data required for a picking operation (GUI).

    APDL Command: FLST

    Parameters
    ----------
    nfield
        Field number on the command which uses the picking data.  (Count
        the command name as a field, so that a 2 indicates the first
        command argument, 3 for the second command argument, etc.)  The
        corresponding field on the command will have a P51X label.

    narg
        Number of items in the picked list.

    type\_
        Type of items picked:

        1 - Node numbers

        2 - Element numbers

        3 - Keypoint numbers

        4 - Line numbers

        5 - Area numbers

        6 - Volume numbers

        7 - Trace points

        8 - Coordinate locations (in Global Cartesian coordinates)

        9 - Screen picks (in X, Y screen coordinates (-1 to 1))

    otype
        Data order:

        NOOR - Data is not ordered (default).

        ORDER - Data is in an ordered list (such as for the E,P51X and A,P51X commands, in
                which the order of the data items is significant for the
                picking operation).

    leng
        Length of number of items describing the list (should equal NARG if
        Otype = NOOR; default).

    Notes
    -----
    Specifies data required for the FITEM command during a picking
    operation.  This is a command generated by the GUI and will appear in
    the log file (Jobname.LOG) if graphical picking is used.  This command
    is not intended to be typed in directly in an ANSYS session (although
    it can be included in an input file for batch input or for use with the
    /INPUT command).

    On the log file, FLST will always be followed by one or more FITEM
    commands which in turn are followed by the ANSYS command that contains
    a P51X label in one of its fields. This set of commands should not be
    edited.

    This command is valid in any processor.
    """
    command = "FLST,%s,%s,%s,%s,%s" % (
        str(nfield), str(narg), str(type_), str(otype), str(leng))
    return self.run(command, **kwargs)



from typing import Optional, Union
from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat


def detab(self, elem="", lab="", v1="", v2="", v3="", v4="", v5="", v6="",
          **kwargs):
    """Modifies element table results in the database.

    APDL Command: DETAB

    Parameters
    ----------
    elem
        Element for which results are to be modified.  If ALL, modify all
        selected elements [ESEL] results.  If ELEM = P, graphical picking
        is enabled and all remaining command fields are ignored (valid only
        in the GUI).  A component name may also be substituted for ELEM.

    lab
        Label identifying results.  Valid labels are as defined with the
        ETABLE command.  Issue ETABLE,STAT to display labels and values.

    v1
        Value assigned to this element table result in the database.  If
        zero, a zero value will be assigned.  If blank, value remains
        unchanged.

    v2, v3, v4, . . . , v6
        Additional values (if any) assigned to consecutive element table
        columns.

    Notes
    -----
    Modifies element table [ETABLE] results in the database.  For example,
    DETAB,35,ABC,1000,2000,1000 assigns 1000, 2000, and 1000 to the first
    three table columns starting with label ABC for element 35.  Use the
    PRETAB command to list the current results.  After deleting a column of
    data using ETABLE,Lab,ERASE, the remaining columns of data are not
    shifted to compress the empty slot.  Therefore, the user must allocate
    null (blank) values for V1, V2...V6 for any ETABLE entries which have
    been deleted by issuing ETABLE,Lab,ERASE.  All data are stored in the
    solution coordinate system but will be displayed in the results
    coordinate system [RSYS].
    """
    command = f"DETAB,{elem},{lab},{v1},{v2},{v3},{v4},{v5},{v6}"
    return self.run(command, **kwargs)


def esort(self, item: str = "", lab: str = "", order: MapdlInt = "",
          kabs: MapdlInt = "", numb: MapdlInt = "",
          **kwargs) -> Optional[str]:
    """Sorts the element table.

    APDL Command: ESORT

    Parameters
    ----------
    item
        Label identifying the item:
        ETAB - (currently the only Item available)

    lab
        element table label: Lab - Any user-defined label from
        the ETABLE command (input in the Lab field of the ETABLE
        command).

    order
        Order of sort operation:

        0 - Sort into descending order.

        1 - Sort into ascending order.

    kabs
        Absolute value key:

        0 - Sort according to real value.

        1 - Sort according to absolute value.

    numb
        Number of elements (element table rows) to be sorted in
        ascending or descending order (ORDER) before sort is
        stopped (remainder will be in unsorted sequence)
        (defaults to all elements).

    Notes
    -----
    The element table rows are sorted based on the column
    containing the Lab values. Use EUSORT to restore the original
    order. If ESORT is specified with PowerGraphics on
    [/GRAPHICS,POWER], then the nodal solution results listing
    [PRNSOL] will be the same as with the full graphics mode
    [/GRAPHICS,FULL].
    """
    command = f"ESORT,{item},{lab},{order},{kabs},{numb}"
    return self.run(command, **kwargs)


def etable(self, lab: str = "", item: str = "", comp: str = "",
           option: str = "", **kwargs) -> Optional[str]:
    """Fills a table of element values for further processing.

    APDL Command: ETABLE

    Parameters
    ----------
    lab
        Any unique user defined label for use in subsequent
        commands and output headings (maximum of eight characters
        and not a General predefined Item label). Defaults to an
        eight character label formed by concatenating the first
        four characters of the Item and Comp labels. If the same
        as a previous user label, this result item will be
        included under the same label. Up to 200 different labels
        may be defined. The following labels are predefined and
        are not available for user-defined labels: ``'REFL''`,
        ``'STAT'``, and ``'ERAS'``.  ``lab='REFL'`` refills all
        tables previously defined with the :meth:`etable` commands
        (not the CALC module commands) according to the latest
        ETABLE specifications and is convenient for refilling
        tables after the load step (SET) has been
        changed. Remaining fields will be ignored if
        ``Lab='REFL'``.  ``lab='STAT'`` displays stored table
        values.  ``lab='ERAS'`` erases the entire table.

    item
        Label identifying the item. General item labels are shown
        in the table below. Some items also require a component
        label. Character parameters may be used. ``item='eras'``
        erases a Lab column.

    comp
        Component of the item (if required). General component
        labels are shown in the table below. Character parameters
        may be used.

    option
        Option for storing element table data:

        * ``'MIN'`` - Store minimum element nodal value of the specified
          item component.
        * ``'MAX'`` - Store maximum element nodal value of the specified
          item component.
        * ``'AVG'`` - Store averaged element centroid value of the
          specified item component (default).

    Examples
    --------
    Print the volume of individual elements.

    >>> mapdl.clear()
    >>> output = mapdl.input(examples.vmfiles['vm6'])
    >>> mapdl.post1()
    >>> label = 'MYVOLU'
    >>> mapdl.etable(label, 'VOLU')
    >>> print(mapdl.pretab(label))
    PRINT ELEMENT TABLE ITEMS PER ELEMENT
       *****ANSYS VERIFICATION RUN ONLY*****
         DO NOT USE RESULTS FOR PRODUCTION
      ***** POST1 ELEMENT TABLE LISTING *****
        STAT     CURRENT
        ELEM     XDISP
           1  0.59135E-001
           2  0.59135E-001
           3  0.59135E-001
    ...

    Notes
    -----
    The ETABLE command defines a table of values per element (the
    element table) for use in further processing. The element
    table is organized similar to spreadsheet, with rows
    representing all selected elements and columns consisting of
    result items which have been moved into the table (Item,Comp)
    via ETABLE. Each column of data is identified by a
    user-defined label (Lab) for listings and displays.

    After entering the data into the element table, you are not
    limited to merely listing or displaying your data (PLESOL,
    PRESOL, etc.). You may also perform many types of operations
    on your data, such as adding or multiplying columns (SADD,
    SMULT), defining allowable stresses for safety calculations
    (SALLOW), or multiplying one column by another (SMULT).  See
    Getting Started in the Basic Analysis Guide for more
    information.

    Various results data can be stored in the element table. For
    example, many items for an element are inherently
    single-valued (one value per element). The single-valued items
    include: SERR, SDSG, TERR, TDSG, SENE, SEDN, TENE, KENE, AENE,
    JHEAT, JS, VOLU, and CENT. All other items are multivalued
    (varying over the element, such that there is a different
    value at each node). Because only one value is stored in the
    element table per element, an average value (based on the
    number of contributing nodes) is calculated for multivalued
    items. Exceptions to this averaging procedure are FMAG and all
    element force items, which represent the sum only of the
    contributing nodal values.

    Two methods of data access can be used with the ETABLE
    command. The method you select depends upon the type of data
    that you want to store.  Some results can be accessed via a
    generic label (Component Name method), while others require a
    label and number (Sequence Number method).

    The Component Name method is used to access the General
    element data (that is, element data which is generally
    available to most element types or groups of element
    types). All of the single-valued items and some of the more
    general multivalued items are accessible with the Component
    Name method.  Various element results depend on the
    calculation method and the selected results location (AVPRIN,
    RSYS, LAYER, SHELL, and ESEL).

    Although nodal data is readily available for listings and
    displays (PRNSOL, PLNSOL) without using the element table, you
    may also use the Component Name method to enter these results
    into the element table for further "worksheet"
    manipulation. (See Getting Started in theBasic Analysis Guide
    for more information.) A listing of the General Item and Comp
    labels for the Component Name method is shown below.

    The Sequence Number method allows you to view results for data
    that is not averaged (such as pressures at nodes, temperatures
    at integration points, etc.), or data that is not easily
    described in a generic fashion (such as all derived data for
    structural line elements and contact elements, all derived
    data for thermal line elements, layer data for layered
    elements, etc.). A table illustrating the Items (such as LS,
    LEPEL, LEPTH, SMISC, NMISC, SURF, etc.) and corresponding
    sequence numbers for each element is shown in the Output Data
    section of each element description found in the Element
    Reference.

    Some element table data are reported in the results coordinate
    system.  These include all component results (for example, UX,
    UY, etc.; SX, SY, etc.). The solution writes component results
    in the database and on the results file in the solution
    coordinate system. When you issue the ETABLE command, these
    results are then transformed into the results coordinate
    system (RSYS) before being stored in the element table. The
    default results coordinate system is global Cartesian
    (RSYS,0).  All other data are retrieved from the database and
    stored in the element table with no coordinate transformation.

    Use the PRETAB, PLETAB, or ETABLE,STAT commands to display the
    stored table values. Issue ETABLE,ERAS to erase the entire
    table. Issue ETABLE,Lab,ERAS to erase a Lab column.

    The element table data option (Option) is not available for
    all output items.

    """
    command = f"ETABLE,{lab},{item},{comp},{option}"
    return self.run(command, **kwargs)


def eusort(self, **kwargs) -> Optional[str]:
    """Restore original order of the element table.

    APDL Command: EUSORT

    Examples
    --------
    >>> mapdl.post1()
    >>> mapdl.eusort()
    'ELEMENT SORT REMOVED'

    Notes
    -----
    Changing the selected element set [ESEL] also restores the original
    element order.
    """
    return self.run("EUSORT", **kwargs)


def pletab(self, itlab="", avglab="", **kwargs):
    """Displays element table items.

    APDL Command: PLETAB

    Parameters
    ----------
    itlab
        User-defined label, as specified with the ETABLE command, of item
        to be displayed.

    avglab
        Averaging operation:

        NOAV - Do not average element items at common nodes (default).

        AVG - Average the element items at common nodes.

    Notes
    -----
    Displays items stored in the table defined with the ETABLE command for
    the selected elements.  For display purposes, items are assumed to be
    constant over the element and assigned to each of its nodes.  Contour
    display lines (lines of constant value) are determined by linear
    interpolation within each element from the nodal values.  These nodal
    values have the option of being averaged (values are averaged at a node
    whenever two or more elements connect to the same node) or not averaged
    (discontinuous).  The discontinuity between contour lines of adjacent
    elements is an indication of the gradient across elements.

    Portions of this command are not supported by PowerGraphics
    [/GRAPHICS,POWER].
    """
    command = f"PLETAB,{itlab},{avglab}"
    return self.run(command, **kwargs)


def plls(self, labi="", labj="", fact="", kund="", viewup="", **kwargs):
    """Displays element table items as contoured areas along elements.

    APDL Command: PLLS

    Parameters
    ----------
    labi
        Label of element table item (ETABLE) for node I magnitude.

    labj
        Label of element table item for node J magnitude.

    fact
        Scale factor for display (defaults to 1).  A negative scaling
        factor may be used to invert the display.

    kund
        Undisplaced shape key:

        0 - Display selected items on undeformed shape.

        1 - Display selected items on deformed shape.

    viewup
        View Up key:

        0 - Ignore the view-up (/VUP) vector when calculating trapezoid orientation
            (default).

        1 - Use the view-up (/VUP) vector to calculate trapezoid orientation.

    Notes
    -----
    Displays selected items (such as shears and moments) as a contoured
    area (trapezoid) display along line elements and 2-D axisymmetric shell
    elements (such as shear and moment diagrams).  Three sides of the
    trapezoid are formed by the element (one side) and lines at nodes I and
    J of length proportional to the item magnitude and displayed normal to
    the element and the viewing direction (the two parallel sides).

    When ViewUP = 1, the trapezoid is oriented within the plane created by
    the element and the global Cartesian coordinate system reference
    orientation (/VUP or view up) vector. In this case, the program does
    not perform the calculation involving the element and view direction.

    Portions of this command are not supported by PowerGraphics
    [/GRAPHICS,POWER].
    """
    command = f"PLLS,{labi},{labj},{fact},{kund},{viewup}"
    return self.run(command, **kwargs)


def pretab(self, lab1="", lab2="", lab3="", lab4="", lab5="", lab6="",
           lab7="", lab8="", lab9="", **kwargs):
    """Prints the element table items.

    APDL Command: PRETAB

    Parameters
    ----------
    lab1, lab2, lab3, . . . , lab9
        Print selected items.  Valid labels are (blank) or any label as
        specified with the ETABLE command.  Convenience labels may be used
        for Lab1 to select groups of labels (10 labels maximum):  GRP1 for
        first 10 stored items; GRP2 for items 11 to 20; GRP3 for items 21
        to 30; GRP4 for items 31 to 40; GRP5 for items 41 to 50.  Enter
        ETABLE,STAT command to list stored item order.  If all labels are
        blank, print first 10 stored items (GRP1).

    Notes
    -----
    Prints the items stored in the table defined with the ETABLE command.
    Item values will be listed for the selected elements in the sorted
    sequence [ESORT].  The FORCE command can be used to define which
    component of the nodal load is to be used (static,  damping, inertia,
    or total).

    Portions of this command are not supported by PowerGraphics
    [/GRAPHICS,POWER].
    """
    command = f"PRETAB,{lab1},{lab2},{lab3},{lab4},{lab5},{lab6},{lab7},{lab8},{lab9}"
    return self.run(command, **kwargs)


def sabs(self, key="", **kwargs):
    """Specifies absolute values for element table operations.

    APDL Command: SABS

    Parameters
    ----------
    key
        Absolute value key:

        0 - Use algebraic values in operations.

        1 - Use absolute values in operations.

    Notes
    -----
    Causes absolute values to be used in the SADD, SMULT, SMAX, SMIN, and
    SSUM operations.
    """
    command = f"SABS,{key}"
    return self.run(command, **kwargs)


def sadd(self, labr="", lab1="", lab2="", fact1="", fact2="", const="",
         **kwargs):
    """Forms an element table item by adding two existing items.

    APDL Command: SADD

    Parameters
    ----------
    labr
        Label assigned to results.  If same as existing label, the existing
        values will be overwritten by these results.

    lab1
        First labeled result item in operation.

    lab2
        Second labeled result item in operation (may be blank).

    fact1
        Scale factor applied to Lab1. A (blank) or '0' entry defaults to
        1.0.

    fact2
        Scale factor applied to Lab2. A (blank) or '0' entry defaults to
        1.0.

    const
        Constant value.

    Notes
    -----
    Forms a labeled result (see ETABLE command) for the selected elements
    by adding two existing labeled result items according to the operation:

    LabR = (FACT1 x Lab1) + (FACT2 x Lab2) + CONST

    May also be used to scale results for a single labeled result item.  If
    absolute values are requested [SABS,1], absolute values of Lab1 and
    Lab2 are used.
    """
    command = f"SADD,{labr},{lab1},{lab2},{fact1},{fact2},{const}"
    return self.run(command, **kwargs)


def sallow(self, strs1="", strs2="", strs3="", strs4="", strs5="",
           strs6="", **kwargs):
    """Defines the allowable stress table for safety factor calculations.

    APDL Command: SALLOW

    Parameters
    ----------
    strs1, strs2, strs3, . . . , strs6
        Input up to six allowable stresses corresponding to the temperature
        points [TALLOW].

    Notes
    -----
    Defines the allowable stress table for safety factor calculations
    [SFACT,SFCALC].  Use the STAT command to list current allowable stress
    table.  Repeat SALLOW to zero table and redefine points (6 maximum).

    Safety factor calculations are not supported by PowerGraphics. Both the
    SALLOW and TALLOW commands must be used with the Full Model Graphics
    display method active.
    """
    command = f"SALLOW,{strs1},{strs2},{strs3},{strs4},{strs5},{strs6}"
    return self.run(command, **kwargs)


def sexp(self, labr="", lab1="", lab2="", exp1="", exp2="", **kwargs):
    """Forms an element table item by exponentiating and multiplying.

    APDL Command: SEXP

    Parameters
    ----------
    labr
        Label assigned to results.  If same as existing label, the existing
        values will be overwritten by these results.

    lab1
        First labeled result item in operation.

    lab2
        Second labeled result item in operation (may be blank).

    exp1
        Exponent applied to Lab1.

    exp2
        Exponent applied to Lab2.

    Notes
    -----
    Forms a labeled result item (see ETABLE command) for the selected
    elements by exponentiating and multiplying two existing labeled result
    items according to the operation:

    LabR = (|Lab1|EXP1) x (|Lab2|EXP2)

    Roots, reciprocals, and divides may also be done with this command.
    """
    command = f"SEXP,{labr},{lab1},{lab2},{exp1},{exp2}"
    return self.run(command, **kwargs)


def sfact(self, type_="", **kwargs):
    """Allows safety factor or margin of safety calculations to be made.

    APDL Command: SFACT

    Parameters
    ----------
    type\_
        Type of calculation:

        0 - No nodal safety factor or margin of safety calculations.

        1 - Calculate and store safety factors in place of nodal stresses.

        2 - Calculate and store margins of safety in place of nodal stresses.

    Notes
    -----
    Allows safety factor (SF) or margin of safety (MS) calculations to be
    made for the average nodal stresses according to:

    SF = SALLOW/|Stress|

    MS = (SALLOW/|Stress|) -- 1.0

    Calculations are done during the display, select, or sort operation (in
    the active coordinate system [RSYS]) with results stored in place of
    the nodal stresses.  Use the PRNSOL or PLNSOL command to display the
    results.

    Note:: : The results are meaningful only for the stress (SIG1, SIGE,
    etc.) upon which SALLOW is based.  Nodal temperatures used are those
    automatically stored for the node.  Related commands are SFCALC,
    SALLOW, TALLOW.
    """
    command = f"SFACT,{type_}"
    return self.run(command, **kwargs)


def sfcalc(self, labr="", labs="", labt="", type_="", **kwargs):
    """Calculates the safety factor or margin of safety.

    APDL Command: SFCALC

    Parameters
    ----------
    labr
        Label assigned to results.  If same as existing label, the existing
        values will be overwritten by these results.

    labs
        Labeled result item corresponding to the element stress.

    labt
        Labeled result item corresponding to the element temperature.

    type\_
        Type of calculation:

        0 or 1 - Use safety factor (SF) calculation.

        2 - Use margin of safety (MS) calculation.

        3 - Use 1/SF calculation.

    Notes
    -----
    Calculates safety factor (SF) or margin of safety (MS) as described for
    the SFACT command for any labeled result item (see ETABLE command) for
    the selected elements.  Use the PRETAB or PLETAB command to display
    results.  Allowable element stress is determined from the SALLOW-TALLOW
    table [SALLOW, TALLOW].
    """
    command = f"SFCALC,{labr},{labs},{labt},{type_}"
    return self.run(command, **kwargs)


def smax(self, labr="", lab1="", lab2="", fact1="", fact2="", **kwargs):
    """Forms an element table item from the maximum of two other items.

    APDL Command: SMAX

    Parameters
    ----------
    labr
        Label assigned to results.  If same as existing label, the existing
        values will be overwritten by these results.

    lab1
        First labeled result item in operation.

    lab2
        Second labeled result item in operation (may be blank).

    fact1
        Scale factor applied to Lab1. A (blank) or '0' entry defaults to
        1.0.

    fact2
        Scale factor applied to Lab2. A (blank) or '0' entry defaults to
        1.0.

    Notes
    -----
    Forms a labeled result item (see ETABLE command) for the selected
    elements by comparing two existing labeled result items according to
    the operation:

    LabR = (FACT1 x Lab1) cmx (FACT2 x Lab2)

    where "cmx" means "compare and save maximum."  If absolute values are
    requested [SABS,1], the absolute values of Lab1 and Lab2 are used.
    """
    command = f"SMAX,{labr},{lab1},{lab2},{fact1},{fact2}"
    return self.run(command, **kwargs)


def smin(self, labr="", lab1="", lab2="", fact1="", fact2="", **kwargs):
    """Forms an element table item from the minimum of two other items.

    APDL Command: SMIN

    Parameters
    ----------
    labr
        Label assigned to results.  If same as existing label, the existing
        values will be overwritten by these results.

    lab1
        First labeled result item in operation.

    lab2
        Second labeled result item in operation (may be blank).

    fact1
        Scale factor applied to Lab1. A (blank) or '0' entry defaults to
        1.0.

    fact2
        Scale factor applied to Lab2. A (blank) or '0' entry defaults to
        1.0.

    Notes
    -----
    Forms a labeled result item (see ETABLE command) for the selected
    elements by comparing two existing labeled result items according to
    the operation:

    LabR = (FACT1 x Lab1) cmn (FACT2 x Lab2)

    where "cmn" means "compare and save minimum."  If absolute values are
    requested [SABS,1], the absolute values of Lab1 and Lab2 are used.
    """
    command = f"SMIN,{labr},{lab1},{lab2},{fact1},{fact2}"
    return self.run(command, **kwargs)


def smult(self, labr="", lab1="", lab2="", fact1="", fact2="", **kwargs):
    """Forms an element table item by multiplying two other items.

    APDL Command: SMULT

    Parameters
    ----------
    labr
        Label assigned to results.  If same as existing label, the existing
        values will be overwritten by these results.

    lab1
        First labeled result item in operation.

    lab2
        Second labeled result item in operation (may be blank).

    fact1
        Scale factor applied to Lab1. A (blank) or '0' entry defaults to
        1.0.

    fact2
        Scale factor applied to Lab2. A (blank) or '0' entry defaults to
        1.0.

    Notes
    -----
    Forms a labeled result item (see ETABLE command) for the selected
    elements by multiplying two existing labeled result items according to
    the operation:

    LabR = (FACT1 x Lab1) x (FACT2 x Lab2)

    May also be used to scale results for a single labeled result item.  If
    absolute values are requested [SABS,1], the absolute values of Lab1 and
    Lab2 are used.
    """
    command = f"SMULT,{labr},{lab1},{lab2},{fact1},{fact2}"
    return self.run(command, **kwargs)


def ssum(self, **kwargs):
    """Calculates and prints the sum of element table items.

    APDL Command: SSUM

    Notes
    -----
    Calculates and prints the tabular sum of each existing labeled result
    item [ETABLE] for the selected elements.  If absolute values are
    requested [SABS,1], absolute values are used.
    """
    command = f"SSUM,"
    return self.run(command, **kwargs)


def tallow(self, temp1="", temp2="", temp3="", temp4="", temp5="",
           temp6="", **kwargs):
    """Defines the temperature table for safety factor calculations.

    APDL Command: TALLOW

    Parameters
    ----------
    temp1, temp2, temp3, . . . , temp6
        Input up to six temperatures covering the range of nodal
        temperatures.  Temperatures must be input in ascending order.

    Notes
    -----
    Defines the temperature table for safety factor calculations [SFACT,
    SALLOW].  Use STAT command to list current temperature table.  Repeat
    TALLOW command to zero table and redefine points (6 maximum).

    Safety factor calculations are not supported by PowerGraphics. Both the
    SALLOW and TALLOW commands must be used with the Full Model Graphics
    display method active.
    """
    command = f"TALLOW,{temp1},{temp2},{temp3},{temp4},{temp5},{temp6}"
    return self.run(command, **kwargs)


def vcross(self, labxr="", labyr="", labzr="", labx1="", laby1="",
           labz1="", labx2="", laby2="", labz2="", **kwargs):
    """Forms element table items from the cross product of two vectors.

    APDL Command: VCROSS

    Parameters
    ----------
    labxr, labyr, labzr
        Label assigned to X, Y, and Z-component of resultant vector.

    labx1, laby1, labz1
        X, Y, and Z-component of first vector label.

    labx2, laby2, labz2
        X, Y, and Z-component of second vector label.

    Notes
    -----
    Forms labeled result items for the selected element from the cross
    product of two vectors:

    {LabXR, LabYR, LabZR} = {LabX1, LabY1, LabZ1} X {LabX2, LabY2, LabZ2}

    Data must be in a consistent coordinate system.  Labels are those
    associated with the ETABLE command.
    """
    command = f"VCROSS,{labxr},{labyr},{labzr},{labx1},{laby1},{labz1},{labx2},{laby2},{labz2}"
    return self.run(command, **kwargs)


def vdot(self, labr="", labx1="", laby1="", labz1="", labx2="", laby2="",
         labz2="", **kwargs):
    """Forms an element table item from the dot product of two vectors.

    APDL Command: VDOT

    Parameters
    ----------
    labr
        Label assigned to dot product result.

    labx1, laby1, labz1
        X, Y, and Z-component of first vector label.

    labx2, laby2, labz2
        X, Y, and Z-component of second vector label.

    Notes
    -----
    Forms labeled result items for the selected element from the dot
    product of two vectors:

    LabR = {LabX1, LabY1, LabZ1} : :  {LabX2, LabY2, LabZ2}

    Data must be in a consistent coordinate system.  Labels are those
    associated with the ETABLE command.
    """
    command = f"VDOT,{labr},{labx1},{laby1},{labz1},{labx2},{laby2},{labz2}"
    return self.run(command, **kwargs)



def aerocoeff(self, aeromodetype="", aeromappedfilenames="",
              aerospecs="", aeroscalar="", nblades="",
              autofileread="", **kwargs):
    """Computes the aero-damping and stiffness coefficients and
    writes them to an APDL array.

    APDL Command: AEROCOEFF

    Parameters
    ----------
    aeromodetype
        Mode type to be used.

        * ``"blade"`` : Non-cyclic cantilevered blade mode (default)

    aeromappedfiles
        Name of string array containing file names of mapped pressures
        from CFD. The file names should be ordered to correspond to
        the AeroSpecs array.

    aerospecs
        Name of numerical array containing data organized to correspond to
        the AeroMappedFiles array.

    aeroscalar
        Scaling value(s) to handle any modal scaling difference
        between structural and CFD modes. The values can be entered as
        a scalar or 1-dimensional array. (each scaling value defaults
        to 1)

    nblades
        Number of blades.

    autofileread
        Key to automatically read and use values from CFD file header.

        *  0 (OFF or NO) : Do not read scaling values or nodal diameter
           from the CFD file header. (default)
        * 1 (ON or YES) : Read scaling values (labeled Mode Multiplier
          in CFD file) from CFD file header. The scaling values read
          will be used in calculations and the AeroScalar input will
          be ignored. The nodal diameter values will be used to cross
          check the value of i (input through AeroSpecs array).

    Notes
    -----
    The AEROCOEFF command is designed to generate an array of
    aerodynamic coefficients that can be used in a cyclic
    mode-superposition harmonic response analysis using the CYCFREQ
    , AERO command to represent aerodynamic stiffness and
    damping. These aerodynamic coefficients can also be used in a damped
    modal analysis phase (CYCFREQ, MODAL) of a cyclic
    mode-superposition harmonic solve. An APDL array called
    JobnameAeroArray is generated using the AEROCOEFF command.  This array
    is compatible with the array needed for the CYCFREQ, AERO command.
    The format of the written array follows that of the CYCFREQ, AERO
    command. The array is formatted as follows:
    ``i, m, n, V_real, V_imag``

    where
    ``i`` = the i th interblade phase angle (IBPA)
    ``m`` = the m th vibrating blade mode
    ``n`` = the n th blade mode generating the pressure oscillations

    ``Vreal`` and ``V_imag`` = the real and imaginary coefficients.

    Prior to issuing the AEROCOEFF command, a non-cyclic cantilevered
    blade modal analysis must be run, either stress-free or
    prestressed using linear perturbation. For more information, see
    Modal Analysis in the Structural Analysis Guide. The file
    requirements for the AEROCOEFF command are the same as those
    needed for modal restart as described in Modal Analysis Restart.
    The AeroSpecs values are specified in a 3×r array *DIM,
    where r is a positive integer equal to the number of interblade
    phase angles and the pressure modes solved for in the CFD
    analysis.

    Each row has the structure: ``i, m, n``
    where
    ``i`` = the i th interblade phase angle (IBPA)
    ``m`` = the m th vibrating blade mode
    ``n`` = the n th blade mode generating the pressure oscillations

    At least one aerodynamic damping coefficient must be specified for
    each IBPA (equal to the number of blades) while keeping and
    constant. If a value is not specified, the program writes an array
    value of zero for both and . The values of and are relative to the
    modes computed in the re- quired modal analysis.

    The number of AeroScalar values must be equal to the number of
    pressure modes ( from Aero- Specs). If the number of AeroScalar
    values is greater than 1, the values must be entered by defining
    an array *DIM and entering the array name in the AeroScalar
    field. For a discussion of how AeroScalar values are computed, see
    Scaling Aerodynamic Coupling Coefficients.

    The value for nBlades should be equal to the number of sectors of
    the system. If there are multiple blades per cyclic sector, then
    the combination of blades on the single sector will have an aero
    coefficient value. In this case, each blade will not have a
    distinct aero coefficient.
    """
    command = f"AEROCOEFF,{aeromodetype},{aeromappedfilenames},{aerospecs},{aeroscalar},{nblades},{autofileread}"
    return self.run(command, **kwargs)


def cint(self, action="", par1="", par2="", par3="", par4="", par5="",
         par6="", par7="", **kwargs):
    """Defines parameters associated with fracture parameter calculations

    APDL Command: CINT

    Parameters
    ----------
    action
        Specifies action for defining or manipulating initial crack data:

        NEW - Initiate a new calculation and assign an ID.

        CTNC - Define the crack tip node component.

        CENC - Define the crack extension node component, the crack tip node, and the crack
               extension direction.

        TYPE - Define the type of calculation to perform.

        DELE - Delete the CINT object associated with the specified ID.

        NCON - Define the number of contours to be calculated in the contour-integral
               calculation.

        SYMM - Indicate whether the crack is on a symmetrical line or plane.

        NORM - Define the crack plane normal.

        UMM - Activate or deactivate the unstructured mesh method.

        EDIR - Crack-assist extension direction.

        PLOT - Plots crack front and crack tip coordinate system.

        CXFE - Define the crack tip element or crack front element set. Valid for XFEM-based
               crack-growth analysis only.

        RADIUS - Define the radius at which the given value is to be evaluated. Valid for XFEM-
                 based crack-growth analysis only.

        RSWEEP - Define the minimum and maximum sweep angle from existing crack direction. Valid
                 for XFEM-based crack-growth analysis only.
    """
    command = f"CINT,{action},{par1},{par2},{par3},{par4},{par5},{par6},{par7}"
    return self.run(command, **kwargs)


def cycfreq(self, option="", value1="", value2="", value3="", value4="",
            value5="", **kwargs):
    """Specifies solution options for a cyclic symmetry mode-superposition

    APDL Command: CYCFREQ
    harmonic analysis.

    Parameters
    ----------
    option
        One of the following options:

        AERO - Specify the array containing the aerodynamic damping coefficients.

        Value1 - The name of the array containing the aerodynamic stiffness damping
                 coefficients.

        BLADE - Blade information required for a mistuning analysis.

        Value1 - The name of the nodal component containing the blade boundary nodes at the
                 blade-to-disk interface. Also include boundary nodes at
                 any shroud interfaces.

        Value2 - The name of the element component containing the blade elements.

        Value3 - The number of blade modes to include in the CMS reduction.

        Value4 - The lower bound of the frequency range of interest. This value is optional.

        Value5 - The upper bound of the frequency range of interest. This value is optional.

        DEFAULT - Set the default cyclic harmonic solution settings.

        EO - Excitation engine order.

        Value1 - The value of the excitation order, which must be an integer. The loadings on
                 the other sectors will be related to the loading on the
                 basic sector based on the engine order phase shift.

        MIST - Mistuning parameters.

        Value1 - The type of mistuning:

        K - Stiffness (frequency) mistuning

        Value2 - The name of the array containing the stiffness mistuning parameters.

        MODAL - Specifies if a damped modal analysis should be performed on the reduced system.

        Value1 - On/Off key.

        0 (OFF or NO) - No modal solution. Perform the harmonic solution.

        1 (ON or YES) - Perform a damped modal analysis of the reduced system in order to obtain the
                        complex frequencies. The harmonic solution is not
                        performed.

        Value2 - Number of modes for the damped modal analysis.

        Value3 - The beginning, or lower end, of the frequency range of interest (in Hz).

        Value4 - The ending, or upper end, of the frequency range of interest (in Hz).

        RESTART - Defines the point at which to restart the harmonic analysis.

        Value1 - The restart point:

        OFF - No restart (default)

        SWEEP - Restart for a new frequency sweep range (HARFRQ)

        MIST - Restart for new mistuning parameters (new mistuning arrays)

        USER - Causes the program to call for a user-defined solution.

        Value1-5 - Values passed down to the user-defined solution.

        STATUS - List the harmonic solution option settings active for the cyclic model.

    Notes
    -----
    The program solves a cyclically symmetric model (set up via the CYCLIC
    command during preprocessing) at the harmonic indices specified via the
    CYCOPT command.

    The aerodynamic coefficients are specified in a 5×(N×r) array (*DIM),
    where N is the number of blades and r can be any positive integer. Each
    column has the structure:

    where:

    One aerodynamic damping coefficient must be specified for each IBPA
    (equal to the number of blades) while keeping m and n constant.

    For constant (frequency-independent) mistuning, the stiffness
    parameters are specified in an N×1 array (*DIM) where N is the number
    of blades.

    For stiffness mistuning, each row entry represents the deviation of
    Young’s modulus from nominal,  (or equivalently, the ratio of the
    frequency deviation squared). Each frequency can also be independently
    mistuned, in which case the array is N×M, where M is the number of
    blade frequencies (Value3 of CYCFREQ,BLADE). The entries in each row
    therefore correspond to the ratio of the mistuned frequency to the
    tuned frequency squared minus one:

    The USER option activates the solution macro CYCMSUPUSERSOLVE.MAC. The
    normal solution is skipped. You may implement your own mistuning
    solution using APDL and APDL Math operations, or call your own program
    for the solution.

    The CYCFREQ command is valid in the preprocessing and solution stages
    of an analysis.

    The CYCFREQ,MODAL,ON command writes modal frequencies to the output
    file. No other postprocessing is available for this modal solve.

    When using CYCFREQ,RESTART, only mistuning parameters or frequency
    range may be changed. All other changes in parameters are ignored.

    To learn more about analyzing a cyclically symmetric structure, see the
    Cyclic Symmetry Analysis Guide.
    """
    command = f"CYCFREQ,{option},{value1},{value2},{value3},{value4},{value5}"
    return self.run(command, **kwargs)


def cyclic(self, nsector="", angle="", kcn="", name="", usrcomp="",
           usrnmap="", **kwargs):
    """Specifies a cyclic symmetry analysis.

    APDL Command: CYCLIC

    Parameters
    ----------
    nsector
        The number of sectors in the full 360 degrees, or one of the
        following options:

        STATUS - Indicates the current cyclic status.

        OFF - Resets model to normal (non-cyclic) status and removes the duplicate sector if
              it exists. This option also deletes automatically detected
              edge components (generated when USRCOMP = 0).

        UNDOUBLE - Removes the duplicate sector if it exists. The duplicate sector is created
                   during the solution (SOLVE) stage of a modal cyclic
                   symmetry analysis.

    angle
        The sector angle in degrees.

    kcn
        An arbitrary reference number assigned to the cyclic coordinate
        system.  The default value of 0 specifies automatic detection.

    name
        The root name of sector low- and high-edge components (line, area,
        or node components). The default root name (when USRCOMP = 0) is
        "CYCLIC". A root name that you specify can contain up to 11
        characters.

    usrcomp
        The number of pairs of user-defined low- and high-edge components
        on the cyclic sector (if any). The default value of 0 specifies
        automatic detection of sector edges; however, the automatic setting
        is not valid in all cases. (For more information, see the Notes
        section below.) If the value is greater than 0, no verification of
        user-defined components occurs.

    usrnmap
        The name of a user-defined array specifying the matching node pairs
        between the sector low and high edges. Valid only when USRCOMP = 0.
        Skips the automatic detection of sector edges. Node pairs may be
        input in any order, but the low edge node must be the first entry
        in each pair.

    Notes
    -----
    You can input your own value for NSECTOR, ANGLE or KCN; if you do so,
    the command verifies argument values before executing.

    When USRCOMP = 0 and USRNMAP = blank (default), the CYCLIC command
    automatically detects low- and high-edge components for models
    comprised of any combination of line, area, or volume elements. If a
    solid model exists, however, the  command uses only the lines, areas,
    and/or volumes to determine the  low- and high-edge components; the
    elements, if any, are ignored.

    Nodes will be automatically rotated unless CYCOPT,USERROT,YES has been
    specified.

    If you issue a CYCOPT,TOLER command to set a tolerance for edge-
    component pairing before issuing the CYCLIC command, the CYCLIC command
    uses the specified tolerance when performing automatic edge-component
    detection.

    For 2-D models, autodetection does not consider the CSYS,5 or CSYS,6
    coordinate system specification.  Autodetection for 180 degree (two-
    sector) models is not possible unless a central hole exists.

    The CYCLIC command sets values and keys so that, if possible, the area-
    mesh (AMESH) or volume-mesh (VMESH) command meshes the sector with
    matching node and element face patterns on the low and high edges. (The
    command has no effect on any other element-creation command.)

    Issue the CYCLIC command prior to the meshing command to, if possible,
    produce a mesh with identical node and element patterns on the low and
    high sector edges. Only the AMESH or VMESH commands can perform
    automated matching. (Other meshing operation commands such as VSWEEP
    cannot.) If you employ a meshing operation other than AMESH or VMESH,
    you should ensure that node and element face patterns match, if
    desired. The CYCLIC command output indicates whether each edge-
    component pair has or can produce a matching node pair.

    A cyclic solution (via the SOLVE command) allows dissimilar mesh
    patterns on the extreme boundaries of a cyclically symmetric model. The
    allowance for dissimilar patterns is useful when you have only finite-
    element meshes for your model but not the geometry data necessary to
    remesh it to obtain identical node patterns. In such cases, it is
    possible to obtain solution results, although perhaps at the expense of
    accuracy. A warning message appears because results may be degraded
    near the sector edges.

    The constraint equations (CEs) that tie together the low and high edges
    of your model are generated at the solution stage of the analysis from
    the low- and high-edge components (and nowhere else). You should verify
    that automatically detected components are in the correct  locations
    and that you can account for all components; to do so, you can list
    (CMLIST) or plot (CMPLOT) the components.

    If you issue the CYCLIC command after meshing and have defined element
    types with rotational degrees of freedom (DOFs), ANSYS generates cyclic
    CEs for rotational DOFs that may not exist on the sector boundaries.
    Issue the CYCOPT,DOF command to prevent unused rotational terms from
    being generated.

    Modal cyclic symmetry analysis is supported by the following
    eigensolvers:

    Block Lanczos (MODOPT,LANB)

    PCG Lanczos (MODOPT,LANPCG)

    Super Node (MODOPT,SNODE)

    Subspace (MODOPT,SUBSP)

    To learn more about analyzing a cyclically symmetric structure, see the
    Cyclic Symmetry Analysis Guide.

    When using the: CYCLIC: command to automatically detect the sector, if
    an area is defined with the: AL: command, the lines need to be oriented
    to form the closed curve.
    """
    command = f"CYCLIC,{nsector},{angle},{kcn},{name},{usrcomp},{usrnmap}"
    return self.run(command, **kwargs)


def cycopt(self, option="", value1="", value2="", value3="", value4="",
           value5="", value6="", value7="", **kwargs):
    """Specifies solution options for a cyclic symmetry analysis.

    APDL Command: CYCOPT

    Parameters
    ----------
    option
        One of the following options:

        BCMULT - Controls whether cyclic sector array parameter names are reused or created new
                 for multiple entities.

        Value1 - The flag value.

        0 (OFF or NO) - Create new array parameter names (default)

        1(ON or YES) - Reuse array parameter names

        COMBINE - For linear static cyclic symmetry analysis with non-cyclically symmetric
                  loading only, expands and combines all harmonic index
                  solutions and writes them to the results file during the
                  solution phase of the analysis.

        Value1 - The flag value.

        0 (OFF or NO) - Disable combining of harmonic index solutions (default)

        1 (ON or YES) - Enable combining of harmonic index solutions

        DEFAULT - Set the default cyclic solution settings.

        DOF - The degrees of freedom to couple from the nodes on the low sector boundary to
              nodes on the high boundary:

        Value1 - The component pair ID number.

        Value2, Value3, Value4,  . . . ,  Value7 - The constraint-equation/-coupling degree of freedom (DOF) for this pair. Repeat
                          the command to add other DOFs. The default is
                          constraint-equation/-coupling all applicable
                          DOFs.

        FACETOL - Tolerance for inclusion of surface nodes into your basic sector. Autodetect
                  defaults to 15°, accommodating most sections. Specify a
                  new Value1 only when extreme cut angles or complex model
                  geometry cause surface nodes to be excluded. See Notes
                  (below) for more information.

        ANSYS, Inc. recommends that successful auto-detection depends more on the value of ANGTOL than the value of FACETOL. Please refer to CYCOPT Auto Detection Tolerance Adjustments for Difficult Cases for more information about auto-detection and the CYCOPT command. - Value1

        The face tolerance applies only to auto detection from node/element models (already meshed and no solid model), and it defaults to 15°.   - HINDEX

        The harmonic index solution ranges for modal or buckling cyclic symmetry analyses. The SOLVE command initiates a cyclic symmetry solution sequence at the harmonic indices specified. (By default, the SOLVE command solves for all available harmonic indices.) Static and harmonic cyclic symmetry solutions always use all harmonic indices required for the applied loads. - EVEN / ODD

        For low-frequency electromagnetic analysis only, EVEN specifies a symmetric solution and ODD specifies an antisymmetric solution.  - The value you specify is based on the harmonic index: EVEN (default) indicates
                          harmonic index = 0, and ODD indicates harmonic
                          index = N / 2 (where N is an integer representing
                          the number of sectors in 360°). A value of ODD
                          applies only when N is an even number.

        The CYCOPT command with this HINDEX option is cumulative. To remove an option (for example, EVEN), issue this command: CYCOPT,HINDEX,EVEN,,,-1 - ALL

        Solve all applicable harmonic indices. - Note:  Value2 must be blank.

        Value1, Value2, Value3 - Solve harmonic indices in range Value1 through Value2 in steps of Value3.
                          Repeat the command to add other ranges. The
                          default solves all applicable harmonic indices.

        Value4 - The only valid value is -1. If specified, it removes Value1 through Value2 in
                 steps of Value3 from the set to solve. By default, if
                 Value4 = -1 then Value1 = 0, Value2 = 0, and Value3 = 1.

        Value5 - For static and harmonic analyses, the tolerance for determining if a Fourier
                 contribution of a load contributes to the response
                 (default = 1.0E-5).

        If Value5=STATIC, it forces the program to solve only the specified harmonic indices (even if a load may have a Fourier contribution in an index not specified). - LDSECT

        Restricts subsequently defined force loads and surface loads to a specified sector. The restriction remains in effect until you change or reset it. This option is not available for harmonic analyses based on mode-superposition (CYCOPT,MSUP,1) - Value1

        The sector number. A value other than 0 (default) is valid for a cyclic symmetry analysis with non-cyclically symmetric loading only. A value of 0 (or ALL) resets the default behavior for cyclic loading (where the loads are identical on all sectors). - MOVE

        Specifies if the program should move high- or low-edge component nodes paired within the specified tolerance (TOLER) to create precisely matching pairs.  - Value1

        The flag value. - 0

        Do not move edge component nodes (default) - 1 or HIGH

        Move the high-edge component nodes to precisely match the low-edge component nodes - -1 or LOW

        Move the low-edge component nodes to precisely match the high-edge component nodes - MSUP

        For modal cyclic symmetry analysis only, this flag is used to limit the results written to the Jobname.MODE and Jobname.RST files in preparation for a subsequent mode-superposition-based analysis. In a linear perturbation modal analysis, this option must be specified in the first load step of the preceding base analysis.  - m         """
    command = f"CYCOPT,{option},{value1},{value2},{value3},{value4},{value5},{value6},{value7}"
    return self.run(command, **kwargs)


def emsym(self, nsect="", **kwargs):
    """Specifies circular symmetry for electromagnetic sources.

    APDL Command: EMSYM

    Parameters
    ----------
    nsect
        The number of circular symmetry sections (defaults to 1).

    Notes
    -----
    Specifies the number of times to repeat electromagnetic sources for
    circular symmetry. Applies to SOURC36 elements and to coupled-field
    elements with electric current conduction results in the database.
    Sources are assumed to be equally spaced over 360° about the global
    Cartesian Z axis.

    This command is also valid in SOLUTION.
    """
    command = f"EMSYM,{nsect}"
    return self.run(command, **kwargs)


def mstole(self, method="", namesurf="", namefluid="", **kwargs):
    """Adds two extra nodes from FLUID116 elements to SURF151 or SURF152

    APDL Command: MSTOLE
    elements for convection analyses.

    Parameters
    ----------
    method
        Mapping method:

        0 - Hybrid method (default).

        1 - Projection method.

        2 - Minimum centroid distance method.

    namesurf
        Component name for a group of SURF151 or SURF152 elements. The name
        must be enclosed in single quotes (e.g., 'COM152') when the MSTOLE
        command is manually typed in.

    namefluid
        Component name for a group of FLUID116 elements. The name must be
        enclosed in single quotes (e.g., 'COM116') when the MSTOLE command
        is manually typed in.

    Notes
    -----
    For convection analyses, the MSTOLE command adds two extra nodes from
    FLUID116 elements to SURF151 or SURF152 elements by employing the
    specified mapping method. In the hybrid method, the projection method
    is tried first and if it fails the centroid distance method is used.
    The SURF151 or SURF152 elements and the FLUID116 elements must be
    grouped into components and named using the CM command.

    The SURF151 or SURF152 extra node option must be set for two extra
    nodes (KEYOPT(5) = 2).

    For more information, see Using the Surface Effect Elements in the
    Thermal Analysis Guide.
    """
    command = f"MSTOLE,{method},{namesurf},{namefluid}"
    return self.run(command, **kwargs)


def perbc2d(self, loc1="", loc2="", loctol="", r1="", r2="", tolr="",
            opt="", plnopt="", **kwargs):
    """Generates periodic constraints for 2-D planar magnetic field analyses.

    APDL Command: PERBC2D

    Parameters
    ----------
    loc1
        Constant coordinate location of the first plane of nodes.  For
        PLNOPT = 1 or 2, the constant coordinate location is the global
        Cartesian coordinate system [CSYS,0] location in the X or Y
        direction respectively.  For PLNOPT = 0, the location is the angle
        in the global cylindrical coordinate system [CSYS,1].

    loc2
        Constant coordinate location of the second plane of nodes.  For
        PLNOPT  = 1 or 2, the constant coordinate location is the global
        Cartesian coordinate system [CSYS,0] location in the X or Y
        direction respectively.  For PLNOPT = 0, the location is the angle
        (in degrees) in the global cylindrical coordinate system [CSYS,1].

    loctol
        Tolerance on the constant coordinate location for node selection.
        Defaults to .00001 for PLNOPT  = 1 or 2 and .001 degrees for PLNOPT
        = 0.

    r1
        Minimum coordinate location along the second plane of nodes.  For
        PLNOPT = 1 or 2, the coordinate location is the global Cartesian
        coordinate system location in the Y or X direction respectively.
        For PLNOPT = 0, the coordinate location is the radial coordinate
        value in the global cylindrical coordinate system.   Periodic
        conditions are not applied to nodes at this location.

    r2
        Maximum coordinate location along the second plane of nodes.  For
        PLNOPT = 1 or 2, the coordinate location is the global Cartesian
        coordinate system location in the Y or X direction respectively.
        For PLNOPT = 0, the coordinate location is the radial coordinate
        value in the global cylindrical coordinate system. Periodic
        conditions are not applied to nodes at this location.

    tolr
        Tolerance dimension on node selection along the plane of nodes.
        Defaults to .00001.

    opt
        Periodic option:

        0 - Odd symmetry (default).  Apply constraint equations such that AZ(i) = -AZ(j).

        1 - Even symmetry.  Apply node coupling such that AZ(i) = AZ(j).

    plnopt
        Symmetry plane option:

        0 - Planes of constant angle in the global cylindrical coordinate system [CSYS,1].

        1 - Planes parallel to the global Cartesian X axis [CSYS,0].

        2 - Planes parallel to the global Cartesian Y axis [CSYS,0].

    Notes
    -----
    PERBC2D invokes an ANSYS macro which generates periodic boundary
    condition constraints for 2-D planar magnetic field analysis.  The
    macro is restricted to node pairs sharing common coordinate values
    along symmetry planes separated by a constant coordinate value.  Planes
    (or lines) must lie at either constant  angles (PLNOPT = 0), constant X
    values (PLNOPT = 1), or constant Y values (PLNOPT = 2).  PERBC2D
    applies constraint equations (OPT = 0, odd symmetry) or node coupling
    (OPT = 1, even symmetry) to each node pair sharing a common coordinate
    value along the symmetry planes.  By default, periodic conditions are
    not applied at the first and last node pairs on the symmetry planes
    unless the input location values, R1 and R2, are adjusted to be less
    than or greater than the actual node coordinate values.  Nodes are
    selected for application of the constraints using the NSEL command with
    tolerances on the constant coordinate location (LOCTOL) and the
    coordinate location along the plane (RTOL).

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"PERBC2D,{loc1},{loc2},{loctol},{r1},{r2},{tolr},{opt},{plnopt}"
    return self.run(command, **kwargs)


def physics(self, option="", title="", fname="", ext="", **kwargs):
    """Writes, reads, or lists all element information

    APDL Command: PHYSICS

    Parameters
    ----------
    option
        Specifies what to do with element information:

        WRITE - Write all appropriate element types, key options,
                real constants, material properties, solution
                analysis options, load step options, constraint
                equations, coupled nodes, defined components, and
                GUI preference settings to the file specified with
                the Fname and Ext arguments.

        READ - Deletes all solution information (material
               properties, solution options, load step options,
               constraint equations, coupled nodes, results, and
               GUI preference settings) then reads all the
               information listed above into the ANSYS database
               from the location specified by the Fname and Ext
               arguments.

        LIST - Lists currently defined physics files and their titles.

        DELETE - Deletes a specified physics file and its title
                 from the database.

        CLEAR - Deletes all material properties, solution options,
                load step options, constraint equations, coupled
                nodes, results, and GUI preference settings from
                the database. Does NOT clear the active physics
                file title from the database.

        STATUS - Displays information about all active elements
                 and settings.

    title
        A user-defined title that quickly identifies a set of
        physics settings.  For example, you might use "Fluid,"
        "Structural," or "Magnetic" as titles.  A title can
        contain up to 64 characters. It can be entered in lower or
        upper case. Lower case is internally converted to upper
        case within the program.

    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Use the PHYSICS command when you are performing a multiphysics
    analysis that involves two different disciplines (for example,
    magnetic and structural analyses) and you cannot solve both
    analyses simultaneously.  Once you have set up physics
    environments for both analyses, you can use the PHYSICS,READ
    command to change between the defined physics environments.
    For more information about doing multiphysics analyses, see
    Sequential Coupled-Field Analysis in the Coupled-Field
    Analysis Guide.

    The PHYSICS command outputs all solution information,
    including analysis options, to the Jobname.PHn file described
    above.  Although it also outputs components, the ANSYS program
    does not list entities (nodes, elements, lines, etc.).

    PHYSICS,WRITE will overwrite existing physics files with the
    same title (even if the name is different). In other words, if
    the directory has a physics file with the same title as the
    active physics file title, but a different name, the
    PHYSICS,WRITE command will overwrite the existing physics file
    and use the existing filename, not the filename specified on
    the PHYSICS,WRITE command.
    """
    command = f"PHYSICS,{option},{title},{fname},{ext}"
    return self.run(command, **kwargs)


def race(self, xc="", yc="", rad="", tcur="", dy="", dz="", cname="",
         **kwargs):
    """Defines a "racetrack" current source.

    APDL Command: RACE

    Parameters
    ----------
    xc
        Location of the mid-thickness of the vertical leg along the working
        plane X-axis.

    yc
        Location of the mid-thickness of the horizontal leg along the
        working plane Y-axis.

    rad
        Radius of curvature of the mid-thickness of the curves in the
        racetrack source.  Defaults to .501 * DY

    tcur
        Total current, amp-turns (MKS), flowing in the source.

    dy
        In-plane thickness of the racetrack source.

    dz
        Out-of-plane thickness (depth) of the racetrack source.

    cname
        An alphanumeric name assigned as a component name to the group of
        SOURC36 elements created by the command macro.  Cname must be
        enclosed in single quotes in the RACE command line.  Cname may be
        up to 32 characters, beginning with a letter and containing only
        letters, numbers, and underscores.  Component names beginning with
        an underscore (e.g., _LOOP) are reserved for use by ANSYS and
        should be avoided.  If blank, no component name is assigned.

    Notes
    -----
    RACE invokes an ANSYS macro which defines a "racetrack" current source
    in the working plane coordinate system.  The current source is
    generated from bar and arc source primitives using the SOURC36 element
    (which is assigned the next available element type number).  The macro
    is valid for use in 3-D magnetic field analysis using a scalar
    potential formulation.  Current flows in a counterclockwise direction
    with respect  to the working plane.

    The diagram below shows you a racetrack current source.
    """
    return self.run(f"RACE,{xc},{yc},{rad},{tcur},{dy},{dz},,,{cname}", **kwargs)


def sstate(self, action="", cm_name="", val1="", val2="", val3="", val4="",
           val5="", val6="", val7="", val8="", val9="", **kwargs):
    """Defines a steady-state analysis.

    APDL Command: SSTATE

    Parameters
    ----------
    action
        Action to perform for defining or manipulating steady-state
        analysis data:

        DEFINE - Define steady-state analysis data

        LIST - List current steady-state analysis data

        DELETE - Delete steady-state analysis data

    cm_name
        Element component name

    val1, ..., val9
        Input values (based on the Action type)

    Notes
    -----
    The SSTATE command specifies steady-state analysis parameters for the
    given element component. The program runs the steady-state analysis if
    the corresponding element key option is enabled for that element
    component.

    The command supports the following elements:

    SOLID185

    3-D 8-Node Structural Solid

    SOLID186

    3-D 20-Node Structural Solid

    SOLID187

    3-D 10-Node Tetrahedral Structural Solid

    SOLSH190

    3-D 8-Node Structural Solid Shell

    Degenerated shape (prism) option not supported.

    SOLID285

    3-D 4-Node Tetrahedral Structural Solid with Nodal Pressures

    For information about steady-state rolling for rebar and solid
    elements, see Steady State Rolling in the Mechanical APDL Theory
    Reference.

    The following data types can be defined:

    SPIN -- Steady-state spinning motion

    TRANSLATE -- Rigid body motion (velocity) that the spinning component
    is undergoing

    Define the steady-state spinning motion:

    SSTATE, DEFINE, CM_Name, SPIN, OMEGA, Method, Val4, Val5, Val6, Val7,
    Val8, Val9

    Spin velocity

    Method to use for defining the spin axis:

    Define the spin axis using two points:

    Val4, Val5, Val6 -- Coordinates of the first point

    Val7, Val8, Val9 -- Coordinates of the second point

    This definition method is currently the only option.

    This command defines a steady state spinning motion of 120 rad/s around
    the spin axis:

    In this case, two points with coordinates (0,0,0) and (0,1,0) define
    the spin axis in the global Y direction.

    Define the rigid body motion (velocity):

    SSTATE, DEFINE, CM_Name, TRANSLATE, Val2, Val3, Val4

    SSTATE, LIST, CM_Name

    Lists all steady-state analysis data defined on the specified element
    component. All data is listed if no component (CM_Name) is specified.

    SSTATE, DELETE, CM_Name

    Deletes all steady-state analysis data defined on the specified element
    component. All data is deleted if no component (CM_Name) is specified.
    """
    command = f"SSTATE,{action},{cm_name},{val1},{val2},{val3},{val4},{val5},{val6},{val7},{val8},{val9}"
    return self.run(command, **kwargs)


def xfdata(self, enrichmentid="", elemnum="", nodenum="", phi="",
           **kwargs):
    """Defines a crack in the model by specifying nodal level set values

    APDL Command: XFDATA

    Parameters
    ----------
    enrichmentid
        Name of the enrichment specified via the associated XFENRICH
        command.

    lsm (or blank)
        Specify LSM to indicate that level set values (PHI and optional
        PSI) are specified.

    elemnum
        Element number.

    nodenum
        Node number associated with the specified element ELNUM.

    phi
        Signed normal distance of the node from the crack.

    Notes
    -----
    Issue the XFDATA command multiple times as needed to specify nodal
    level set values for all nodes of an element.

    This command is valid in PREP7 (/PREP7) only.
    """
    command = f"XFDATA,{enrichmentid},{elemnum},{nodenum},{phi}"
    return self.run(command, **kwargs)


def xfenrich(self, enrichmentid="", compname="", matid="", **kwargs):
    """Defines parameters associated with crack propagation using XFEM

    APDL Command: XFENRICH

    Parameters
    ----------
    enrichmentid
        An alphanumeric name assigned to identify the enrichment. The name
        can contain up to 32 characters and must begin with an alphabetic
        character. Alphabetic characters, numbers, and underscores are
        valid.

    compname
        Name of the element set component for which initial cracks are
        defined and possibly propagated.

    matid
        Material ID number referring to cohesive zone material behavior on
        the initial crack. If 0 or not specified, the initial crack is
        assumed to be free of cohesive zone behavior.

    Notes
    -----
    If MatID is specified, the cohesive zone behavior is described by the
    bilinear cohesive law.

    This command is valid in PREP7 (/PREP7) only.
    """
    command = f"XFENRICH,{enrichmentid},{compname},{matid}"
    return self.run(command, **kwargs)


def xflist(self, enrichmentid="", **kwargs):
    """Lists enrichment details and associated crack information

    APDL Command: XFLIST

    Parameters
    ----------
    enrichmentid or (blank)
        Name of the enrichment specified via the associated XFENRICH
        command. Specifying EnrichmentID is optional.

    Notes
    -----
    This command is valid in PREP7 (/PREP7) and SOLUTION (/SOLU).
    """
    command = f"XFLIST,{enrichmentid}"
    return self.run(command, **kwargs)



def cat5in(self, name="", extension="", path="", entity="", fmt="",
           nocl="", noan="", **kwargs):
    """Transfers a .CATPart file into the ANSYS program.

    APDL Command: ~CAT5IN

    Parameters
    ----------
    name
        The name of a valid .CATPart file, created with CATIA Version 5.0.
        The first character of the file name must be an alphanumeric.

    extension
        The extension for the file. The default extension is .CATPart.

    path
        The path name of the directory in which the file resides enclosed
        in single quotes. The default path name is the current working
        directory.

    entity
        Entity to be imported.

        SOLIDS - Solids only, imported as ANSYS volumes (default).

        SURFACES - Surfaces only, imported as ANSYS areas.

        ALL - All entities. Use this option when the file contains different types of
              entities.

    fmt
        The format in which ANSYS will store the model.

        0 - Neutral format (default). Defeaturing after import is restricted.

        1 - Solid format; this allows defeaturing after import.

    nocl
        Remove tiny objects.

        0 - Remove tiny objects without checking model validity (default).

        1 - Do not remove tiny objects.

    noan
        Perform an analysis of the model.

        0 - Analyze the model (default).

        1 - Do not analyze the model.

    Notes
    -----
    If defeaturing is specified (FMT = 1), this command must be the last
    line of any file, script, or other interactive input.

    More information on importing CATIA Version 5 parts is available in
    CATIA V5 in the Connection User's Guide.
    """
    command = f"~CAT5IN,{name},{extension},{path},{entity},{fmt},{nocl},{noan}"
    return self.run(command, **kwargs)


def catiain(self, name="", extension="", path="", blank="", **kwargs):
    """Transfers a CATIA model into the ANSYS program.

    APDL Command: ~CATIAIN

    Parameters
    ----------
    name
        The name of a valid CATIA model, created with CATIA 4.x or
        lower.  The first character of the file name must be an
        alphanumeric.  Special characters such as & - and * and
        spaces are not permitted in the part name.

    extension
        The extension for the file. The default extension is .model.

    path
        The path name of the directory in which the file resides, enclosed
        in single quotes. The default path name is the current working
        directory.

    blank
        Sets whether to import "blanked" entities.

        0 - Does not import "blanked" (suppressed) CATIA entities (default).

        1 - Imports "blanked" entities. The portions of CATIA data
        that were suppressed will be included in the import.

    Notes
    -----
    More information on importing CATIA parts is available in CATIA V4 in
    the Connection User's Guide.
    """
    return self.run(f"~CATIAIN,{name},{extension},{path},,,{blank}", **kwargs)


def parain(self, name="", extension="", path="", entity="", fmt="",
           scale="", **kwargs):
    """Transfers a Parasolid file into the ANSYS program.

    APDL Command: ~PARAIN

    Parameters
    ----------
    name
        The name of a valid Parasolid file. The first character of the file
        name must be an alphanumeric.

    extension
        The extension for the file. The default extension is .x_t on a PC
        or .xmt_txt on a Linux system. Parasolid files are compatible
        across systems, and do not need to be renamed to be used on another
        platform.

    path
        The path name of the directory in which the file resides, enclosed
        in single quotes. The default path name is the current working
        directory.

    entity
        Entity to be imported:

        SOLIDS - Solids only, imported as ANSYS volumes (default)

        SURFACES - Surfaces only, imported as ANSYS areas.

        WIREFRAME - Wireframe only, imported as ANSYS lines.

        ALL - All entities. Use this option when the file contains
              more than one type of entity.

    fmt
        Sets the format in which ANSYS will store the model

        0 - Neutral format (default). Defeaturing after import is
            restricted. Use this option if you need to scale a model
            to a specific unit of measure (other than meters).

        1 - Solid format; this allows defeaturing after import.

    scale
        Allows scaling for the model

        0 - Do not rescale the model; retain the default Parasolid
            setting of meters (default).

        1 - Scale the model if warranted by the model size.

    Notes
    -----
    More information on importing Parasolid parts is available in Parasolid
    in the Connection User's Guide.
    """
    command = f"~PARAIN,{name},{extension},{path},{entity},{fmt},{scale}"
    return self.run(command, **kwargs)


def proein(self, name="", extension="", path="", proecomm="", **kwargs):
    """APDL Command: ~PROEIN

    Transfers a Creo Parametric part into the ANSYS program.

    Parameters
    ----------
    name
        The name of the Creo Parametric part to be imported, which cannot
        exceed 64 characters in length and must begin with an alphanumeric
        character. Special characters such as & - and * and spaces are not
        permitted in the part name.

    extension
        The general Creo Parametric extension format is prt for parts and
        asm for assemblies.

    path
        Full path name to the directory containing the part. The default is
        the current working directory.

    proecomm
        The start command for the version of Creo Parametric you are using.
        proe1 is the default command. Note that the full path name to the
        Creo Parametric command need not be used here if the path had been
        included in the PATH variable. The Creo Parametric command name is
        set by the PROE_START_CMD162 environment variable.

    Notes
    -----
    More information on importing Creo Parametric parts is available in
    Creo Parametric (formerly Pro/ENGINEER) in the Connection User's Guide.
    """
    command = f"~PROEIN,{name},{extension},{path},{proecomm}"
    return self.run(command, **kwargs)


def satin(self, name="", extension="", path="", entity="", fmt="", nocl="",
          noan="", **kwargs):
    """Transfers a .SAT file into the ANSYS program.

    APDL Command: ~SATIN

    Parameters
    ----------
    name
        The name of a valid .SAT file, created with a supported version of
        ACIS. The first character of the file name must be an alphanumeric.
        Special characters such as & - and * and spaces are not permitted
        in the part name. See File Names in the Command Reference for more
        information about ANSYS file naming conventions.

    extension
        The extension for the file. The default extension is .sat.

    path
        The path name of the directory in which the file resides enclosed
        in single quotes. The default path name is the current working
        directory.

    entity
        Entity to be imported.

        SOLIDS - Solids only, imported as ANSYS volumes (Not implemented, imports All).

        SURFACES - Surfaces only, imported as ANSYS areas (Not implemented, imports All).

        WIREFRAME - Wireframe only, imported as ANSYS lines (Not implemented, imports All).

        ALL - All entities. Use this option when the file contains different types of
              entities.

    fmt
        The format in which ANSYS will store the model.

        0 - Neutral format (default). Defeaturing after import is restricted.

        1 - Solid format; this allows defeaturing after import.

    nocl
        Remove tiny objects.

        0 - Remove tiny objects without checking model validity (default).

        1 - Do not remove tiny objects.

    noan
        Perform an ACIS analysis of the model.

        0 - Analyze the model (default).

        1 - Do not analyze the model.

    Notes
    -----
    More information on importing ACIS parts is available in ACIS in the
    Connection User's Guide.
    """
    command = f"~SATIN,{name},{extension},{path},{entity},{fmt},{nocl},{noan}"
    return self.run(command, **kwargs)


def ugin(self, name="", extension="", path="", entity="", layer="", fmt="",
         **kwargs):
    """Transfers an NX part into the ANSYS program.

    APDL Command: ~UGIN

    Parameters
    ----------
    name
        The file name of the NX part to be imported, which cannot exceed 64
        characters in length. The path name must begin with an alphanumeric
        character. Special characters such as &, -,  and * are not
        permitted in the part name.

    extension
        The NX part file extension. The default is .prt.

    path
        The full path name to the directory containing the part, enclosed
        in single quotes; for example, '/ug_parts'. The default is the
        current working directory.

    entity
        Entity to be imported.

        0 or Solid - Solids only, imported as ANSYS volumes (the
        default).

        1 or Surface - Surfaces only, imported as ANSYS areas.

        2 or Wireframe - Wireframe only, imported as ANSYS lines.

        3 or All - All entities. Use this option when the part
                   contains entities that may not be attached to each
                   other, such as a solid in one location and a
                   surface in another.

    layer
        The number(s) assigned to the layer(s) to be imported. You can
        import one layer or a range of layers (designated by hyphens).
        Defaults to 1-256 (all layers).

    fmt
        The format in which ANSYS will store the model.

        0 - Neutral format (default). Defeaturing after import is restricted.

        1 - Solid format; this allows defeaturing after import.

    Notes
    -----
    More information on importing NX parts is available in UG/NX in the
    Connection User's Guide.
    """
    command = f"~UGIN,{name},{extension},{path},{entity},{layer},{fmt}"
    return self.run(command, **kwargs)



def mfconv(self, lab="", toler="", minref="", **kwargs):
    """Sets convergence values for an ANSYS Multi-field solver analysis.

    APDL Command: MFCONV

    Parameters
    ----------
    lab
        Valid labels:

    toler
        Convergence tolerance about program calculated reference value (the
        L2 norm of the new load in a multi-field analysis). Defaults to
        0.01 (1%) for all labels. Must be less than 1.0.

    minref
        The minimum value allowed for the program calculated reference
        value. If negative, no minimum is enforced. Defaults to 1.0e-6 for
        all labels. Not available in the GUI. MINREF corresponds to
        ||ϕnew|| as defined in Set up Stagger Solution in the Coupled-Field
        Analysis Guide.

    Notes
    -----
    MFCONV sets convergence values for variables at the ANSYS
    Multi-field solver interface.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide
    for a list of all ANSYS Multi-field solver commands and their
    availability for MFS and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported
    in Distributed ANSYS.
    """
    return self.run(f"MFCONV,{lab},{toler},,{minref}", **kwargs)


def mfiter(self, maxiter="", miniter="", target="", **kwargs):
    """Sets the number of stagger iterations for an ANSYS Multi-field solver

    APDL Command: MFITER
    analysis.

    Parameters
    ----------
    maxiter
        Maximum number of iterations. Defaults to 10.

    miniter
        Minimum number of iterations. Defaults to 1.

    target
        Target number of iterations. Defaults to 5.

    Notes
    -----
    The number of stagger iterations applies to each time step in an ANSYS
    Multi-field solver analysis. MINITER and TARGET are valid only when
    multi-field auto time stepping is on (MFDTIME).

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFITER,{maxiter},{miniter},{target}"
    return self.run(command, **kwargs)


def mfrelax(self, lab="", value="", option="", **kwargs):
    """Sets relaxation values for an ANSYS Multi-field solver analysis.

    APDL Command: MFRELAX

    Parameters
    ----------
    lab
        Valid labels:

    value
        Relaxation value. Defaults to 0.75 for all labels.

    option
        Valid options are:

        RELX  - Uses relaxation method for load transfer (default).

        LINT  - Uses a linear interpolation for loaf transfer.

    Notes
    -----
    MFRELAX sets relaxation values for the load transfer variables at a
    surface or volume interface. Option = RELX will usually give you a more
    stable and smooth load transfer and is suitable for strongly coupled
    problems (such as FSI problems). Option = LINT is suitable for weakly
    coupled problems because it will transfer the full load in fewer
    stagger iterations.

    See the MFFR and MFITER commands for more information on relaxation in
    the ANSYS Multi-field solver.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFRELAX,{lab},{value},{option}"
    return self.run(command, **kwargs)



def m(self, node="", lab1="", nend="", ninc="", lab2="", lab3="", lab4="",
      lab5="", lab6="", **kwargs):
    """Defines master degrees of freedom for superelement generation analyses.

    APDL Command: M

    Parameters
    ----------
    node
        Node number at which master degree of freedom is defined.  If ALL,
        define master degrees of freedom at all selected nodes (NSEL).  If
        NODE = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NODE.

    lab1
        Valid degree of freedom label. If ALL, use all appropriate labels.
        Structural labels:  UX, UY, or UZ (displacements); ROTX, ROTY, or
        ROTZ (rotations). Thermal labels: TEMP, TBOT, TE2, TE3, . . ., TTOP
        (temperature).  Electric labels:  VOLT (voltage).

    nend, ninc
        Define all nodes from NODE to NEND (defaults to NODE) in steps of
        NINC (defaults to 1) as master degrees of freedom in the specified
        direction.

    lab2, lab3, lab4, . . . , lab6
        Additional master degree of freedom labels.  The nodes defined are
        associated with each label specified.

    Notes
    -----
    Defines master degrees of freedom (MDOF) for superelement generation.
    If defined for other analyses, MDOF are ignored.  If used in SOLUTION,
    this command is valid only within the first load step.

    Repeat M command for additional master degrees of freedom.  The limit
    for the number of master nodes used is determined by the maximum system
    memory available.

    The substructure (ANTYPE,SUBSTR) analysis utilizes the matrix
    condensation technique to reduce the structure matrices to those
    characterized by a set of master degrees of freedom.

    Master degrees of freedom are identified by a list of nodes and their
    nodal directions.  The actual degree of freedom directions available
    for a particular node depends upon the degrees of freedom associated
    with element types (ET) at that node.  There must be some mass (or
    stress stiffening in the case of the buckling analysis) associated with
    each master degree of freedom (except for the VOLT label).  The mass
    may be due either to the distributed mass of the element or due to
    discrete lumped masses at the node.  If a master degree of freedom is
    specified at a constrained point, it is ignored.  If a master degree of
    freedom is specified at a coupled node, it should be specified at the
    prime node of the coupled set.

    Substructure analysis connection points must be defined as master
    degrees of freedom.

    This command is also valid in PREP7.
    """
    command = f"M,{node},{lab1},{nend},{ninc},{lab2},{lab3},{lab4},{lab5},{lab6}"
    return self.run(command, **kwargs)


def mdele(self, node="", lab1="", nend="", ninc="", lab2="", lab3="",
          lab4="", lab5="", lab6="", **kwargs):
    """Deletes master degrees of freedom.

    APDL Command: MDELE

    Parameters
    ----------
    node, lab1, nend, ninc
        Delete master degrees of freedom in the Lab1 direction [M] from
        NODE to NEND (defaults to NODE) in steps of NINC (defaults to 1).
        If NODE = ALL, NEND and NINC are ignored and masters for all
        selected nodes [NSEL] are deleted.  If Lab1 = ALL, all label
        directions will be deleted.  If NODE = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for NODE.

    lab2, lab3, lab4, . . . , lab6
        Delete masters in these additional directions.

    Notes
    -----
    Deletes master degrees of freedom.  If used in SOLUTION, this command
    is valid only within the first load step.

    This command is also valid in PREP7.
    """
    command = f"MDELE,{node},{lab1},{nend},{ninc},{lab2},{lab3},{lab4},{lab5},{lab6}"
    return self.run(command, **kwargs)


def mgen(self, itime="", inc="", node1="", node2="", ninc="", **kwargs):
    """Generates additional MDOF from a previously defined set.

    APDL Command: MGEN

    Parameters
    ----------
    itime, inc
        Do this generation operation a total of ITIMEs, incrementing all
        nodes in the set by INC each time after the first.  ITIME must be >
        1 for generation to occur.  All previously defined master degree of
        freedom directions are included in the set.  A component name may
        also be substituted for ITIME.

    node1, node2, ninc
        Generate master degrees of freedom from set beginning with NODE1 to
        NODE2 (defaults to NODE1) in steps of NINC (defaults to 1).  If
        NODE1 = ALL, NODE2 and NINC are ignored and set is all selected
        nodes [NSEL].  If NODE1 = P, graphical picking is enabled and all
        remaining command fields are ignored (valid only in the GUI).

    Notes
    -----
    Generates additional master degrees of freedom from a previously
    defined set.  If used in SOLUTION, this command is valid only within
    the first load step.

    This command is also valid in PREP7.
    """
    command = f"MGEN,{itime},{inc},{node1},{node2},{ninc}"
    return self.run(command, **kwargs)


def mlist(self, node1="", node2="", ninc="", **kwargs):
    """Lists the MDOF of freedom.

    APDL Command: MLIST

    Parameters
    ----------
    node1, node2, ninc
        List master degrees of freedom from NODE1 to NODE2 (defaults
        toNODE1) in steps of NINC (defaults to 1).  If NODE1 = ALL
        (default), NODE2 and NINC are ignored and masters for all selected
        nodes [NSEL] are listed.  If NODE1 = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for NODE1
        (NODE2 and NINC are ignored).

    Notes
    -----
    Lists the master degrees of freedom.
    """
    command = f"MLIST,{node1},{node2},{ninc}"
    return self.run(command, **kwargs)



def atype(self, **kwargs):
    """Specifies "Analysis types" as the subsequent status topic.

    APDL Command: ATYPE

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"ATYPE,"
    return self.run(command, **kwargs)


def bioopt(self, **kwargs):
    """Specifies "Biot-Savart options" as the subsequent status topic.

    APDL Command: BIOOPT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"BIOOPT,"
    return self.run(command, **kwargs)


def deact(self, **kwargs):
    """Specifies "Element birth and death" as the subsequent status topic.

    APDL Command: DEACT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"DEACT,"
    return self.run(command, **kwargs)


def dynopt(self, **kwargs):
    """Specifies "Dynamic analysis options" as the subsequent status topic.

    APDL Command: DYNOPT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"DYNOPT,"
    return self.run(command, **kwargs)


def gap(self, **kwargs):
    """Specifies "mode-superposition transient gap conditions" as the

    APDL Command: GAP
    subsequent status topic.

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"GAP,"
    return self.run(command, **kwargs)


def genopt(self, **kwargs):
    """Specifies "General options" as the subsequent status topic.

    APDL Command: GENOPT

    Notes
    -----
    This is a status (STAT) topic command. Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"GENOPT,"
    return self.run(command, **kwargs)


def inrtia(self, **kwargs):
    """Specifies "Inertial loads" as the subsequent status topic.

    APDL Command: INRTIA

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"INRTIA,"
    return self.run(command, **kwargs)


def lsoper(self, **kwargs):
    """Specifies "Load step operations" as the subsequent status topic.

    APDL Command: LSOPER

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"LSOPER,"
    return self.run(command, **kwargs)


def master(self, **kwargs):
    """Specifies "Master DOF" as the subsequent status topic.

    APDL Command: MASTER

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"MASTER,"
    return self.run(command, **kwargs)


def nlopt(self, **kwargs):
    """Specifies "Nonlinear analysis options" as the subsequent status topic.

    APDL Command: NLOPT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"NLOPT,"
    return self.run(command, **kwargs)


def outopt(self, **kwargs):
    """Specifies "Output options" as the subsequent status topic.

    APDL Command: OUTOPT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"OUTOPT,"
    return self.run(command, **kwargs)


def smbody(self, **kwargs):
    """Specifies "Body loads on the solid model" as the subsequent status

    APDL Command: SMBODY
    topic.

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"SMBODY,"
    return self.run(command, **kwargs)


def smcons(self, **kwargs):
    """Specifies "Constraints on the solid model" as the subsequent status

    APDL Command: SMCONS
    topic.

    Notes
    -----
    This is a status [STAT] topic command. Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"SMCONS,"
    return self.run(command, **kwargs)


def smfor(self, **kwargs):
    """Specifies "Forces on the solid model" as the subsequent status topic.

    APDL Command: SMFOR

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"SMFOR,"
    return self.run(command, **kwargs)


def smsurf(self, **kwargs):
    """Specifies "Surface loads on the solid model" as the subsequent status

    APDL Command: SMSURF
    topic.

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"SMSURF,"
    return self.run(command, **kwargs)


def soluopt(self, **kwargs):
    """Specifies "Solution options" as the subsequent status topic.

    APDL Command: SOLUOPT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"SOLUOPT,"
    return self.run(command, **kwargs)


def sptopt(self, **kwargs):
    """Specifies "Spectrum analysis options" as the subsequent status topic.

    APDL Command: SPTOPT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"SPTOPT,"
    return self.run(command, **kwargs)



def define(self, **kwargs):
    """Specifies "Data definition settings" as the subsequent status topic.

    APDL Command: DEFINE

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"DEFINE,"
    return self.run(command, **kwargs)


def operate(self, **kwargs):
    """Specifies "Operation data" as the subsequent status topic.

    APDL Command: OPERATE

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"OPERATE,"
    return self.run(command, **kwargs)


def plotting(self, **kwargs):
    """Specifies "Plotting settings" as the subsequent status topic.

    APDL Command: PLOTTING

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"PLOTTING,"
    return self.run(command, **kwargs)


def print(self, **kwargs):
    """Specifies "Print settings" as the subsequent status topic.

    APDL Command: PRINT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"PRINT,"
    return self.run(command, **kwargs)



def fk(self, kpoi="", lab="", value="", value2="", **kwargs):
    """Defines force loads at keypoints.

    APDL Command: FK

    Parameters
    ----------
    kpoi
        Keypoint at which force is to be specified.  If ALL, apply to all
        selected keypoints [KSEL].  If KPOI = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for KPOI.

    lab
        Valid force label.  Structural labels:  FX, FY, or FZ (forces); MX,
        MY, or MZ (moments).  Thermal labels:  HEAT, HBOT, HE2, HE3, . . .,
        HTOP (heat flow).  Fluid labels:  FLOW (fluid flow).  Electric
        labels:  AMPS (current flow), CHRG (electric charge).  Magnetic
        labels:  FLUX (magnetic flux);  CSGX, CSGY, or CSGZ (magnetic
        current segments). Diffusion labels: RATE (diffusion flow rate).

    value
        Force value or table name reference for specifying tabular boundary
        conditions.  To specify a table, enclose the table name in percent
        signs (%), e.g., FK, KPOI, HEAT,%tabname%).  Use the *DIM command
        to define a table.

    value2
        Second force value (if any).  If the analysis type and the force
        allow a complex input, VALUE (above) is the real component and
        VALUE2 is the imaginary component.

    Notes
    -----
    Forces may be transferred from keypoints to nodes with the FTRAN or
    SBCTRAN commands.  See the F command for a description of force loads.

    Tabular boundary conditions (VALUE = %tabname%) are available only for
    the following labels: Fluid (FLOW), Electric (AMPS), Structural force
    (FX, FY, FZ, MX, MY, MZ), and Thermal (HEAT, HBOT, HE2, HE3, . . .,
    HTOP).

    This command is also valid in PREP7.
    """
    command = f"FK,{kpoi},{lab},{value},{value2}"
    return self.run(command, **kwargs)


def fkdele(self, kpoi="", lab="", **kwargs):
    """Deletes force loads at a keypoint.

    APDL Command: FKDELE

    Parameters
    ----------
    kpoi
        Keypoint at which force is to be deleted.  If ALL, delete forces at
        all selected keypoints [KSEL].  If KPOI = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for KPOI.

    lab
        Valid force label.  If ALL, use all appropriate labels.  See the
        FDELE command for labels.

    Notes
    -----
    Deletes force loads (and all corresponding finite element loads) at a
    keypoint.  See the FDELE command for details.

    This command is also valid in PREP7.
    """
    command = f"FKDELE,{kpoi},{lab}"
    return self.run(command, **kwargs)


def fklist(self, kpoi="", lab="", **kwargs):
    """Lists the forces at keypoints.

    APDL Command: FKLIST

    Parameters
    ----------
    kpoi
        List forces at this keypoint.  If ALL (default), list for all
        selected keypoints [KSEL].  If KPOI = P, graphical picking is
        enabled and all remaining command fields are ignored (valid only in
        the GUI).  A component name may also be substituted for KPOI.

    lab
        Force label to be listed (defaults to ALL).  See the DOFSEL command
        for labels.

    Notes
    -----
    Listing applies to the selected keypoints [KSEL] and the selected force
    labels [DOFSEL].

    This command is valid in any processor.
    """
    command = f"FKLIST,{kpoi},{lab}"
    return self.run(command, **kwargs)


def ftran(self, **kwargs):
    """Transfers solid model forces to the finite element model.

    APDL Command: FTRAN

    Notes
    -----
    Forces are transferred only from selected keypoints to selected nodes.
    The FTRAN operation is also done if the SBCTRAN command is issued or
    automatically done upon initiation of the solution calculations
    [SOLVE].

    This command is also valid in PREP7.
    """
    command = f"FTRAN,"
    return self.run(command, **kwargs)



def mfcalc(self, fnumb="", freq="", **kwargs):
    """Specifies a calculation frequency for a field in an ANSYS Multi-field

    APDL Command: MFCALC
    solver analysis.

    Parameters
    ----------
    fnumb
        Field number set by the MFELEM command.

    freq
        Perform calculation every Nth ANSYS Multi-field solver time step.
        Defaults to 1 for every time step.

    Notes
    -----
    This command only applies to a harmonic analysis of the specified
    field. It is useful when a field contributes negligible field
    interaction within a single ANSYS Multi-field solver time step.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFCALC,{fnumb},{freq}"
    return self.run(command, **kwargs)


def mfdtime(self, dtime="", dtmin="", dtmax="", carry="", **kwargs):
    """Sets time step sizes for an ANSYS Multi-field solver analysis.

    APDL Command: MFDTIME

    Parameters
    ----------
    dtime
        Multi-field time step size.  If automatic time stepping is being
        used [see Notes below], DTIME is the starting time step.

    dtmin
        Minimum time step. Defaults to DTIME.

    dtmax
        Maximum time step. Defaults to DTIME.

    carry
        Time step carryover key.

        OFF  - Use DTIME as the starting time step for the next restart run (default).

        ON  - Use the final time step from the previous run as the starting time step for the
              next restart run.

    Notes
    -----
    This command specifies time step sizes for an ANSYS Multi-field solver
    analysis. If either DTMIN or DTMAX is not equal to DTIME, auto time-
    stepping is turned on for the multi-field loop. ANSYS will
    automatically adjust the time step size for the next multi-field step
    between DTMIN and DTMAX, based on the status of the current
    convergence, the number of target stagger iterations (specified by
    MFITER), and the actual number of iterations needed to reach
    convergence at the current time step.

    If auto time-stepping is turned off, the time step size must be evenly
    divisible into the end time (specified by MFTIME) minus the start time
    (0 for a new analysis or a restart time specified by MFRSTART).

    You can use a smaller time step within each ANSYS field analysis. This
    is called subcycling. Use the DELTIM and AUTOTS commands to subcycle a
    structural, thermal, or electromagnetic analysis.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFDTIME,{dtime},{dtmin},{dtmax},{carry}"
    return self.run(command, **kwargs)


def mfoutput(self, freq="", **kwargs):
    """Specifies results file output frequency for an ANSYS

    APDL Command: MFOUTPUT
    Multi-field solver analysis.

    Parameters
    ----------
    freq
        N

        N - Write solution every Nth (and the last) time
             step. Defaults to 1, for every time step.

        -N - Writes up to -N equally spaced results (for multifield auto time stepping).

        NONE - Suppresses writing of results for all multifield time steps.

        ALL - Writes results for every multifield time step (default).

        LAST - Writes results for only the last multifield time step.

        %array% - Where %array% is the name of an n X 1 X 1
                  dimensional array parameter defining n key
                  times, the data for the specified solution
                  results item is written at those key times. Key
                  times in the array parameter must appear in
                  ascending order. Value must be greater than or
                  equal to the ending time values for the load
                  step.

        For restart runs (see MFRSTART command), either change the parameter values to fall between the beginning and ending time values of the load step, or erase the current settings and reissue the command with a new array parameter.  - For more information about defining array parameters, see the *DIM command
                          documentation.

    Notes
    -----
     A MFOUTPUT setting overrides any other output frequency setting
    (OUTRES). To select the solution items, use the OUTRES command.

    For the case of Freq = -n and Freq = %array%, the results at the time
    points which first time reaches or exceeds the targeting ouptupt time
    points will be written.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFOUTPUT,{freq}"
    return self.run(command, **kwargs)


def mfrstart(self, time="", **kwargs):
    """Specifies restart status for an ANSYS Multi-field solver analysis.

    APDL Command: MFRSTART

    Parameters
    ----------
    time
        Restart time

        0  -  New analysis (Default)

        -1  - Restart from the last result set from a previous run.

        n  - Specify any positive number for the actual time point from which the ANSYS
             Multi-field solver will restart. ANSYS checks the availability
             of the result set and database file.

    Notes
    -----
    For MFX analyses, ANSYS always passes an actual time value to CFX (zero
    for a new analysis or a positive value for a restart run) and CFX
    verifies the consistency with the initial results file. For more
    details about ANSYS restart capabilities, please see Restarting an
    Analysis in the Basic Analysis Guide.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFRSTART,{time}"
    return self.run(command, **kwargs)


def mftime(self, time="", **kwargs):
    """Sets end time for an ANSYS Multi-field solver analysis.

    APDL Command: MFTIME

    Parameters
    ----------
    time
        End time of an ANSYS Multi-field solver analysis. Defaults to 1.

    Notes
    -----
    A MFTIME setting overrides any other end time setting (TIME).

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFTIME,{time}"
    return self.run(command, **kwargs)



def mfanalysis(self, key="", **kwargs):
    """Activates or deactivates an ANSYS Multi-field solver analysis.

    APDL Command: MFANALYSIS

    Parameters
    ----------
    key
        Multifield analysis key:

        ON - Activates an ANSYS Multi-field solver analysis.

        OFF - Deactivates an ANSYS Multi-field solver analysis (default).

    Notes
    -----
    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFANALYSIS,{key}"
    return self.run(command, **kwargs)


def mfclear(self, option="", value="", **kwargs):
    """Deletes ANSYS Multi-field solver analysis settings.

    APDL Command: MFCLEAR

    Parameters
    ----------
    option
        SOLU

        SOLU - Resets all ANSYS solution commands except KBC to their default states. This
               option clears analysis options when setting up different
               fields for an ANSYS Multi-field solver analysis.

        FIELD - Deletes all ANSYS Multi-field solver specifications for the specified field
                number.

        SINT - Deletes all ANSYS Multi-field solver specifications for the specified surface
               interface number.

        VINT - Deletes all ANSYS Multi-field solver specifications for the volumetric
               interface number.

        ORD - Deletes the analysis order specified by the MFORDER command.

        EXT - Deletes external fields specified by the MFEXTER command

        MFLC - Deletes load transfers specified by the MFLCOMM command

    value
        Use only for Option = FIELD, SINT, or VINT.

    Notes
    -----
    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFCLEAR,{option},{value}"
    return self.run(command, **kwargs)


def mffr(self, fname="", lab="", rfini="", rfmin="", rfmax="", **kwargs):
    """Setup Multi-Field relaxation factors for field solutions.

    APDL Command: MFFR

    Parameters
    ----------
    fname
         Field name (MFX) or number (MFS). Must be the ANSYS field (cannot
        be a CFX field).

    lab
        Label name. Valid values are DISP and TEMP.

    rfini
        Initial relaxation factor. Defaults to 0.75.

    rfmin
        Minimum relaxation factor. Defaults to RFINI.

    rfmax
        Maximum relaxation factor. Defaults to RFINI.

    Notes
    -----
    Use this command to relax the field solutions in fluid-solid
    interaction analyses and thermal-thermal analyses for a better
    convergence rate in coupled problems, especially cases that need
    dynamic relaxation. The ANSYS field that has the MFFR command applied
    will do only one nonlinear stagger iteration within each multi-field
    stagger; the convergence of the ANSYS field solver will be satisfied
    through multiple multi-field staggers. Note that the CFX field solver
    can have multiple iterations within the field solver; see the CFX
    documentation for more details. ANSYS will not terminate the nonlinear
    field solution until the ANSYS field solver converges or reaches the
    maximum number of multi-field staggers as specified on MFITER.

    The interface load relaxation (MFRELAX) will be automatically turned
    off for the corresponding surface loads that have MFFR applied. The
    automatic change of the relaxation factor for accelerating the
    nonlinear convergence of the coupled field solution is based on
    Aitken's acceleration scheme.

    This command is valid only with coupled problems involving surface load
    transfer only. No subcycling is allowed for the field solver if using
    this command.
    """
    command = f"MFFR,{fname},{lab},{rfini},{rfmin},{rfmax}"
    return self.run(command, **kwargs)


def mfinter(self, option="", **kwargs):
    """Specifies the interface load transfer interpolation option for an ANSYS

    APDL Command: MFINTER
    Multi-field solver analysis.

    Parameters
    ----------
    option
        Interface load transfer option:

        CONS - Conservative formulation for load transfer.

        NONC - Nonconservative formulation for load transfer (default).

    Notes
    -----
    This command only applies to the interpolation method for forces, heat
    flux, and heat generation. Displacement and temperature transfers are
    always nonconservative.

    For more information on conservative and nonconservative load transfer,
    see Load Transfer in the Coupled-Field Analysis Guide.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFINTER,{option}"
    return self.run(command, **kwargs)


def mflist(self, option="", value="", **kwargs):
    """Lists the settings for an ANSYS Multi-field solver analysis.

    APDL Command: MFLIST

    Parameters
    ----------
    option
        ALL

        ALL - Lists all ANSYS Multi-field solver analysis options.

        SOLU - Lists all solution-related ANSYS Multi-field solver options.

        FIELD - Lists all ANSYS Multi-field solver options related to the specified field
                number.

        SINT - Lists all surface interface information for the specified surface interface
               number.

        VINT - Lists all volumetric interface information for the specified volumetric
               interface number.

    value
        Use only for Option = FIELD, SINT, or VINT.

    Notes
    -----
    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFLIST,{option},{value}"
    return self.run(command, **kwargs)


def mforder(self, fnumb1="", fnumb2="", fnumb3="", fnumb4="", fnumb5="",
            fnumb6="", fnumb7="", fnumb8="", fnumb9="", fnumb10="",
            fnumb11="", fnumb12="", fnumb13="", fnumb14="", fnumb15="",
            fnumb16="", fnumb17="", fnumb18="", fnumb19="", fnumb20="",
            **kwargs):
    """Specifies field solution order for an ANSYS Multi-field solver

    APDL Command: MFORDER
    analysis.

    Parameters
    ----------
    fnumb1, fnumb2, fnumb3, . . . , fnumb20
        Field numbers defined by the MFELEM command.

    Notes
    -----
    You can define up to twenty fields in an ANSYS Multi-field solver
    analysis.

    This command is also valid in PREP7.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFORDER,{fnumb1},{fnumb2},{fnumb3},{fnumb4},{fnumb5},{fnumb6},{fnumb7},{fnumb8},{fnumb9},{fnumb10},{fnumb11},{fnumb12},{fnumb13},{fnumb14},{fnumb15},{fnumb16},{fnumb17},{fnumb18},{fnumb19},{fnumb20}"
    return self.run(command, **kwargs)


def mfpsimul(self, gname="", fname1="", fname2="", **kwargs):
    """Sets up a field solver group to simultaneously process with code

    APDL Command: MFPSIMUL
    coupling analyses.

    Parameters
    ----------
    gname
        Sets the group name with a character string of up to 80 characters.

    fname1, fname2
        Sets the field solver 1 and field solver 2 names, which are
        processed simultaneously, with a character string of up to 80
        characters.

    Notes
    -----
    This command is used to define a group of simultaneously-processed
    field solvers in an MFX analysis. For example, to define group g1 with
    field solvers ansys-code and cfx-code, enter MFPS,g1,ansys-code,cfx-
    code.

    To indicate groups of sequentially-processed field solvers for your MFX
    analysis, create two groups (g1 and g2).

    A field solver refers to a specific instance of an ANSYS or CFX solver
    execution that is defined by the respective input file(s) referenced
    when starting the solver (through the launcher or from the command
    line). The field solver names that are referenced in several MFX
    commands must be consistent with the names that will be used when
    starting the coupled simulation.

    Note:: : When running MFX from the launcher, you must use ANSYS and CFX
    (uppercase) as the field solver names (MFPSIMUL) in your input file.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFPSIMUL,{gname},{fname1},{fname2}"
    return self.run(command, **kwargs)


def mfsorder(self, gname1="", gname2="", **kwargs):
    """Sets up the solution sequence of simultaneous field solver groups for

    APDL Command: MFSORDER
    code coupling analyses.

    Parameters
    ----------
    gname1, gname2
        Specifies the group name for groups defined by the MFPSIMUL command
        with a character string of up to 80 characters.

    Notes
    -----
    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFSORDER,{gname1},{gname2}"
    return self.run(command, **kwargs)


def mfwrite(self, fname="", ext="", **kwargs):
    """Writes an ANSYS master input file for MFX multiple code coupling.

    APDL Command: MFWRITE

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum, including the
        characters needed for the directory path).  An unspecified
        directory path defaults to the working directory; in this case, you
        can use all 248 characters for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    When working interactively, you need to issue this command as the last
    step in your setup process. This command will write out the input file
    that you will then use to submit the MFX analysis. This file will
    include the /SOLU, SOLVE, and FINISH commands.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.
    """
    command = f"MFWRITE,{fname},{ext}"
    return self.run(command, **kwargs)
